<?php

/*

FormHandler v3
 
Revision: $Date: 2005/10/02 19:48:23 $

Look for more info at http://www.formhandler.net

*/

// make sure this file is not accessed directly
if(strtolower(basename($_SERVER['PHP_SELF'])) == strtolower(basename(__FILE__))) {
    die("This file cannot be accessed directly! Include it in your script instead!");
}

/******* BUILD IN VALIDATOR FUNCTIONS ****/
define('FH_STRING',     'IsString',    true);	// any string that doesn't have control characters (ASCII 0 - 31) but spaces are allowed
define('FH_ALPHA',      'IsAlpha',     true);	// only letters a-z and A-Z
define('FH_DIGIT',      'IsDigit',     true);	// only numbers 0-9
define('FH_ALPHA_NUM',  'IsAlphaNum',  true);	// letters and numbers
define('FH_INTEGER',    'IsInteger',   true);	// only numbers 0-9 and an optional - (minus) sign (in the beginning only)
define('FH_FLOAT',      'IsFloat',     true);	// like FH_INTEGER, only with , (comma)
define('FH_FILENAME',   'IsFilename',  true);	// a valid file name (including dots but no slashes and other forbidden characters)
define('FH_BOOL',       'IsBool',      true);	// a boolean (TRUE is either a case-insensitive "true" or "1". Everything else is FALSE)
define('FH_VARIABLE',   'IsVariabele', true);	// a valid variable name (letters, digits, underscore)
define('FH_PASSWORD',   'IsPassword',  true);	// a valid password (alphanumberic + some other characters but no spaces. Only allow ASCII 33 - 126)
define('FH_URL',        'IsURL',       true);	// a valid URL (http connection is used to check if url exists!)
define('FH_EMAIL',      'IsEmail',     true);	// a valid email address (only checks for valid format: xxx@xxx.xxx)
define('FH_EMAIL_HOST', 'IsEmailHost', true);   // like FH_EMAIL only with host check
define('FH_TEXT',       'IsText',      true);	// like FH_STRING, but newline characters are allowed
define('FH_NOT_EMPTY',  'notEmpty',    true);   // check if the value is not empty
define('FH_NO_HTML',	'NoHTML',	   true);   // check if the value does not contain html
define('FH_IP',		    'IsIp',		   true);   // check if the value is a valid ip adres (xxx.xxx.xxx.xxx:xxxx)		

# for dutch people
define('FH_POSTCODE',   'IsPostcode',  true);   // valid dutch postcode (eg. 9999 AA)
define('FH_PHONE',      'IsPhone',     true);   // valid dutch phone-number(eg. 058-2134778)

# same as above, but with these the value is not required
define('_FH_STRING',     '_IsString',    true);
define('_FH_ALPHA',      '_IsAlpha',     true);
define('_FH_DIGIT',      '_IsDigit',     true);
define('_FH_ALPHA_NUM',  '_IsAlphaNum',  true);
define('_FH_INTEGER',    '_IsInteger',   true);
define('_FH_FLOAT',      '_IsFloat',     true);	
define('_FH_FILENAME',   '_IsFilename',  true);
define('_FH_BOOL',       '_IsBool',      true);
define('_FH_VARIABLE',   '_IsVariabele', true);
define('_FH_PASSWORD',   '_IsPassword',  true);
define('_FH_URL',        '_IsURL',       true);
define('_FH_EMAIL',      '_IsEmail',     true);
define('_FH_EMAIL_HOST', '_IsEmailHost', true);
define('_FH_TEXT',       '_IsText',      true);
define('_FH_POSTCODE',   '_IsPostcode',  true);
define('_FH_PHONE',      '_IsPhone',     true);
define('_FH_NO_HTML',	 '_NoHTML',	     true);
define('_FH_IP',		 '_IsIp',		 true);


// make some variables global when the version < 4.1.0
if(IntVal( str_replace('.', '', phpversion()) ) < 410) {
    define('_global', false);
    $_GET =& $HTTP_GET_VARS;
    $_POST =& $HTTP_POST_VARS;
    $_FILES =& $HTTP_POST_FILES;
    $_SERVER =& $HTTP_SERVER_VARS;
} else {
    define('_global', true);
}

// includes
define('FH_INCLUDE_DIR', dirname(__FILE__).'/');
require_once(FH_INCLUDE_DIR.'fields/class.Field.php');
require_once(FH_INCLUDE_DIR.'buttons/class.Button.php');
require_once(FH_INCLUDE_DIR.'includes/config.inc.php');
require_once(FH_INCLUDE_DIR.'includes/error.inc.php');
require_once(FH_INCLUDE_DIR.'includes/class.validator.php');
require_once(FH_INCLUDE_DIR.'includes/class.handler.php');


/**
 * class FormHandler
 *
 * Create a new FormHandler object. 
 *
 */
class FormHandler extends Handler {
    // public
    var $insert;            // boolean: if the form is an insert-form
    var $edit;              // boolean: if the form is an edit-form 
    
    /**
     * FormHandler::FormHandler()
     *
     * Public constructor: create a new instance of FormHandler
     *
     * @param string $name: the name of the form
     * @return FormHandler
     */
    function FormHandler($name = null, $action = null, $extra = null) { 
        // make vars global if needed
        if(!_global) global $_SERVER, $_POST, $_GET;
        $this->Handler();
              
    	// try to disable caching from the browser if possible
    	if(!headers_sent()) {
    		header('Expires: Mon, 26 Jul 1997 05:00:00 GMT'); 
			header('Last-Modified: ' . gmdate('D, d M Y H:i:s') . ' GMT');		
			header('Cache-Control: no-store, no-cache, must-revalidate');  
			header('Cache-Control: post-check=0, pre-check=0', false);
			header('Pragma: no-cache');
    	}
 	
        // set all config values
        fh_conf();
        
        // set the name of the form
        if(!empty($name)) {
            $this->_name = $name;
        } else {            
        	// get a unique form name!
            $i = null;
            while(defined('FH_'.FH_DEFAULT_FORM_NAME.$i)) {
                $i = (is_null($i)) ? 1 : ($i+1);
            }            
            define('FH_'.FH_DEFAULT_FORM_NAME.$i, 1);
            $this->_name = FH_DEFAULT_FORM_NAME.$i;
            $i = null;
        }
        
        // set the action of the form if none is given
        if( !empty($action) ) {
            $this->_action = $action;
        } else {
            $this->_action = $_SERVER['PHP_SELF'];
            if(!empty($_SERVER['QUERY_STRING'])) {
                $this->_action .= '?'.$_SERVER['QUERY_STRING'];
            }
        }      
        
        // get the JS to put into the <form> tag
        if( !empty( $extra ) ) {
        	$this->_extra = $extra;
        }      
        
        // initialisation 
        $this->insert = !isset($_GET[FH_EDIT_NAME]);
        $this->edit   = !$this->insert;         
        
        // get the ID if it's an edit form           
        if($this->edit) {
            $this->_id = $_GET[FH_EDIT_NAME];
            if( !is_array($this->_id) ) {
            	$this->_id = array( $this->_id );
            }
        }
        
        // set the default mask
        $this->setMask( FH_DEFAULT_ROW_MASK );
        
        // set the default help icon
        $this->setHelpIcon( FH_FHTML_DIR.'images/helpicon.gif' );
        
        // check if the form is posted
        $this->_posted = ($_SERVER['REQUEST_METHOD'] == 'POST' && isset($_POST[$this->_name.'_submit']));
        
        // make a hidden field so we can identify the form
        $this->hiddenField($this->_name.'_submit', '1');
        
        // get the current page
        $this->_curPage = isset($_POST[$this->_name.'_page']) ? $_POST[$this->_name.'_page'] : 1;
        
        // set our own error handler
        if(FH_DISPLAY_ERRORS) {
        	error_reporting( E_ALL );
            set_error_handler('catchErrors');
        }
        
        // set the language...
        $this->setLanguage();        
    }  
    
    /******************/
    /***** FIELDS *****/  
    /******************/
    
    /**
     * FormHandler::textField()
     *
     * Public: creates a textfield on the form
     *
     * @param string $title: The title of the field
     * @param string $name: The name of the field
     * @param string $validator: The validator which should be used to validate the value of the field
     * @param int $size: The size of the field
     * @param int $maxlength: The allowed max input of the field
     * @param string $extra: CSS, Javascript or other which are inserted into the HTML tag
     * @return void
     */
    function textField(
      $title, 
      $name, 
      $validator = null, 
      $size      = null, 
      $maxlength = null, 
      $extra     = null) 
    {
    	require_once(FH_INCLUDE_DIR.'fields/class.TextField.php');

        // create the field
        $fld =& new TextField($this, $name);
        if(!is_null($validator)) $fld->setValidator($validator);
        if(!is_null($size))      $fld->setSize($size);
        if(!is_null($maxlength)) $fld->setMaxlength($maxlength);
        if(!is_null($extra))     $fld->setExtra($extra);
        
        // add the field to the fields array
        $this->_fields[$name] = array($title, &$fld);
    }   
    
    /**
     * FormHandler::passField()
     *
     * Public: Create a password field
     *
     * @param string $title: The title of the field
     * @param string $name: The name of the field
     * @param string $validator: The validator which should be used to validate the value of the field
     * @param int $size: The size of the field
     * @param int $maxlength: The allowed max input of the field
     * @param string $extra: CSS, Javascript or other which are inserted into the HTML tag
     * @return void
     */
    function passField(
      $title, 
      $name, 
      $validator = null, 
      $size      = null, 
      $maxlength = null, 
      $extra     = null) 
    {
    	require_once(FH_INCLUDE_DIR.'fields/class.TextField.php');
    	require_once(FH_INCLUDE_DIR.'fields/class.PassField.php');
    	
        // create the field
        $fld =& new PassField( $this, $name );
        if(!is_null($validator)) $fld->setValidator($validator);
        if(!is_null($size))      $fld->setSize($size);
        if(!is_null($maxlength)) $fld->setMaxlength($maxlength);
        if(!is_null($extra))     $fld->setExtra($extra);
        
        // add the field to the fields array
        $this->_fields[$name] = array($title, &$fld);
    } 
    
    /**
     * FormHandler::hiddenField()
     *
     * Public: create a hidden field
     *
     * @param string $name: The name of the field
     * @param string $value: The value of the field     
     * @param string $validator: The validator which should be used to validate the value of the field
     * @param string $extra: CSS, Javascript or other which are inserted into the HTML tag
     * @return void
     */
    function hiddenField(
      $name, 
      $value     = null, 
      $validator = null,
      $extra     = null)
    {
    	require_once(FH_INCLUDE_DIR.'fields/class.HiddenField.php');
    	
        // create new hidden field
        $fld =& new HiddenField($this, $name);
        
        // only set the hidden field value if there is not a value in the $_POST array
        if(!is_null($value) && !$this->isPosted() )     
                                 $fld->setValue($value);
        if(!is_null($validator)) $fld->setValidator($validator);
        if(!is_null($extra))     $fld->setExtra($extra);
       
        // add the field to the fields array
        $this->_fields[$name] = array('__HIDDEN__',  &$fld);
    }
    
    /**
     * FormHandler::textArea()
     *
     * Public: Create a textarea on the form
     *
     * @param string $title: The title of the field
     * @param string $name: The name of the field
     * @param string $validator: The validator which should be used to validate the value of the field
     * @param int $cols: How many cols (the width of the field)
     * @param int $rows: How many rows (the height of the field) 
     * @param string $extra: CSS, Javascript or other which are inserted into the HTML tag
     * @return void
     */ 
    function textArea(
      $title,
      $name,
      $validator = null,
      $cols      = null,
      $rows      = null, 
      $extra     = null)
    {
    	require_once(FH_INCLUDE_DIR.'fields/class.TextArea.php');

        // create new textarea
        $fld =& new TextArea($this, $name);
        if(!is_null($validator)) $fld->setValidator($validator);
        if(!is_null($cols))      $fld->setCols($cols);
        if(!is_null($rows))      $fld->setRows($rows);
        if(!is_null($extra))     $fld->setExtra($extra);
        
        // add the field to the fields array
        $this->_fields[$name] = array($title, &$fld);
    }     
    
    /**
     * FormHandler::selectField()
     *
     * Public: Create a selectField on the form
     *
     * @param string $title: The title of the field
     * @param string $name: The name of the field
     * @param array $options: The options used for the field
     * @param string $validator: The validator which should be used to validate the value of the field
     * @param boolean $useArrayKeyAsValue: If the array key's are the values for the options in the field
     * @param int $size: The size of the field (how many options are displayed)
     * @param boolean $multiple: Should it be possible to select multiple options ? (Default: false)
     * @param string $extra: CSS, Javascript or other which are inserted into the HTML tag
     * @return void
     */ 
    function selectField(
      $title,
      $name,
      $options,
      $validator          = null,
      $useArrayKeyAsValue = null,
      $multiple           = null,
      $size               = null,      
      $extra              = null)
    {
    	require_once(FH_INCLUDE_DIR.'fields/class.SelectField.php');

        // options has to be an array
        if(!is_array($options)) {
            trigger_error(
              "You have to give an array as value with the selectfield '$name'",
              E_USER_WARNING
            );
            return;
        }
        
        // create new selectfield
        $fld =& new SelectField( $this, $name );
        $fld->setOptions($options);
        if(!is_null($validator))          $fld->setValidator( $validator );
        if(!is_null($useArrayKeyAsValue)) $fld->useArrayKeyAsValue( $useArrayKeyAsValue );
        if(!is_null($extra))              $fld->setExtra( $extra );
        if(!is_null($multiple))           $fld->setMultiple( $multiple );        
        if(!is_null($size)) {              
            $fld->setSize( $size );
        // if no size is set and multiple is enabled, set the size default to 4    
        } else if( !is_null($multiple) ) {
            $fld->setSize( 4 );
        }
        
        
        // add the field to the fields array
        $this->_fields[$name] = array( $title, &$fld );
    }
    
    
    /**
     * FormHandler::dbSelectField()
     *
     * Public: Create a selectField on the form with records loaded from a table
     *
     * @param string $title: The title of the field
     * @param string $name: The name of the field
     * @param string $table: The table where the records are retrieved from
     * @param mixed $fields: String or array with the field(s) which are retrieved from the table and put into the select field
     * @param string $validator: The validator which should be used to validate the value of the field
     * @param boolean $multiple: Should it be possible to select multiple options ? (Default: false)
     * @param int $size: The size of the field (how many options are displayed)
     * @param string $extra: CSS, Javascript or other which are inserted into the HTML tag
     * @param array $mergeArray: Default value(s) for the field
     * @return void
     */ 
    function dbSelectField( 
      $title, 
      $name, 
      $table, 
      $fields,
      $extraSQL   = null, 
      $validator  = null,
      $multiple   = null,
      $size       = null,
      $extra      = null,
      $mergeArray = null) 
    {
    	require_once(FH_INCLUDE_DIR.'fields/class.SelectField.php');
    	require_once(FH_INCLUDE_DIR.'fields/class.dbSelectField.php');

        if( !is_object($this->_db) ) {
            trigger_error(
              'Error, you have to make use of the database option to make use of the field dbSelectField. '.
              'Use selectField() instead!', 
              E_USER_WARNING
            );
            return;
        } 
        
        // create new selectfield
        $fld =& new dbSelectField( $this, $name, $this->_db, $table, $fields, $extraSQL, $mergeArray );
        if(!is_null($validator))          $fld->setValidator($validator);
        if(!is_null($multiple))           $fld->setMultiple( $multiple );        
        if(!is_null($size)) {              
            $fld->setSize( $size );
        // if no size is set and multiple is enabled, set the size default to 4    
        } else if( !is_null($multiple) ) {
            $fld->setSize( 4 );
        }
        if(!is_null($extra))              $fld->setExtra($extra);        
        
        // add the field to the fields array
        $this->_fields[$name] = array($title, &$fld);        
    }
    
    /**
     * FormHandler::checkBox()
     *
     * Public: Create a checkBox on the form
     *
     * @param string $title: The title of the field
     * @param string $name: The name of the field
     * @param array|string $value: The option(s) used for the field
     * @param string $validator: The validator which should be used to validate the value of the field
     * @param boolean $useArrayKeyAsValue: If the array key's are the values for the options in the field
     * @param string $extra: CSS, Javascript or other which are inserted into the HTML tag
     * @param string $glue: if more the 1 options are given, glue the fields together with this
     * @return void
     */ 
    function checkBox(
      $title,
      $name,
      $value              = 'on',
      $validator          = null,
      $useArrayKeyAsValue = null,
      $extra              = null,
      $glue               = null)
    {
    	require_once(FH_INCLUDE_DIR.'fields/class.CheckBox.php');

        // create a new checkbox
        $fld =& new CheckBox($this, $name, $value);
        if(!is_null($validator))          $fld->setValidator($validator);
        if(!is_null($useArrayKeyAsValue)) $fld->useArrayKeyAsValue($useArrayKeyAsValue);
        if(!is_null($extra))              $fld->setExtra($extra);
        if(!is_null($glue))               $fld->setGlue($glue);
        
        // add the field to the fields array
        $this->_fields[$name] = array($title, &$fld);
    }
    
    /**
     * FormHandler::radioButton()
     *
     * Public: Create a radioButton on the form
     *
     * @param string $title: The title of the field
     * @param string $name: The name of the field
     * @param array $options: The options used for the field
     * @param string $validator: The validator which should be used to validate the value of the field
     * @param boolean $useArrayKeyAsValue: If the array key's are the values for the options in the field
     * @param string $extra: CSS, Javascript or other which are inserted into the HTML tag
     * @param string $glue: if more the 1 options are given, glue the fields together with this
     * @return void
     */
    function radioButton(
      $title,
      $name,
      $options,
      $validator          = null,
      $useArrayKeyAsValue = null,
      $extra              = null,
      $glue               = null)
    {
    	require_once(FH_INCLUDE_DIR.'fields/class.RadioButton.php');

        // value has to be an array
        if(!is_array($options)) {
            trigger_error(
              "You have to give an array as value with the radiobutton '$name'",
              E_USER_WARNING
            );
            return;
        }
           
        // create a new checkbox
        $fld =& new RadioButton($this, $name, $options);
        if(!is_null($validator))          $fld->setValidator($validator);
        if(!is_null($useArrayKeyAsValue)) $fld->useArrayKeyAsValue($useArrayKeyAsValue);
        if(!is_null($extra))              $fld->setExtra($extra);
        if(!is_null($glue))               $fld->setGlue($glue);
        
        // add the field to the fields array
        $this->_fields[$name] = array($title, &$fld);
    }
    
    /**
     * FormHandler::uploadField()
     *
     * Public: Create a uploadField on the form
     *
     * @param string $title: The title of the field
     * @param string $name: The name of the field
     * @param array $config: The configuration used for the field
     * @param string $validator: The validator which should be used to validate the value of the field
     * @param string $extra: CSS, Javascript or other which are inserted into the HTML tag
     * @param string $alertOverwrite: Do we have to alert the user when he/she is going to overwrite a file?
     * @return void
     */
    function uploadField(
      $title,
      $name, 
      $config         = array(),
      $validator      = null,
      $extra          = null,
      $alertOverwrite = null) 
    {
    	require_once(FH_INCLUDE_DIR.'fields/class.UploadField.php');
        
        $fld =& new UploadField($this, $name, $config);
        if(!is_null($validator))      $fld->setValidator($validator);
        if(!is_null($extra))          $fld->setExtra($extra);
        if(!is_null($alertOverwrite)) $fld->setAlertOverwrite($alertOverwrite);
        
        // add the uploadfield to the fields array
        $this->_fields[$name] = array( $title, &$fld );
        
        // set that this form is using uploadfields
        $this->_upload[] = $name;
    }
    
    /**
     * FormHandler::listField()
     *
     * Public: Create a listField on the form
     *
     * @param string $title: The title of the field
     * @param string $name: The name of the field
     * @param array $options: The options used for the field
     * @param string $validator: The validator which should be used to validate the value of the field
     * @param string $onTitle: The title used above the ON section of the field
     * @param string $offTitle: The title used above the OFF section of the field          
     * @param boolean $useArrayKeyAsValue: If the array key's are the values for the options in the field
     * @param int $size: The size of the field (how many options are displayed)
     * @param string $extra: CSS, Javascript or other which are inserted into the HTML tag
     * @return void
     */ 
    function listField(
      $title,
      $name,
      $options,
      $validator          = null,
      $useArrayKeyAsValue = null,
      $onTitle            = null,
      $offTitle           = null,
      $size               = null,
      $extra              = null)
    {
    	require_once(FH_INCLUDE_DIR.'fields/class.SelectField.php');
    	require_once(FH_INCLUDE_DIR.'fields/class.ListField.php');

        // options has to be an array
        if(!is_array($options)) {
            trigger_error(
              "You have to give an array as value with the listfield '$name'",
              E_USER_WARNING
            );
            return;
        }
        
        // create a listfield
        $fld =& new ListField( $this, $name, $options );
        if(!is_null($validator))          $fld->setValidator( $validator );
        if(!is_null($useArrayKeyAsValue)) $fld->useArrayKeyAsValue( $useArrayKeyAsValue );
        if(!empty($size))                 $fld->setSize( $size );
        if(!is_null($extra))              $fld->setExtra( $extra ); 
        if(!empty($onTitle))			  $fld->setOnTitle( $onTitle );
        if(!empty($offTitle))             $fld->setOffTitle( $offTitle );	
        
        // add the listfield to the fields array
        $this->_fields[$name] = array($title, &$fld);
    } 
    
    /**
     * FormHandler::editor()
     *
     * Public: Create a editor on the form
     *
     * @param string $title: The title of the field
     * @param string $name: The name of the field
     * @param string $validator: The validator which should be used to validate the value of the field
     * @param string $path: Path on the server where we have to upload the files     
     * @param string $toolbar: The toolbar we have to use
     * @param string $skin: The skin to use
     * @param int $width: The width of the field         
     * @param int $height: The height of the field         
     * @param boolean $useArrayKeyAsValue: If the array key's are the values for the options in the field
     * @param string $extra: CSS, Javascript or other which are inserted into the HTML tag
     * @return void
     */ 
    function editor(
      $title,
      $name, 
      $validator = null,
      $path      = null,
      $toolbar   = null,
      $skin      = null,
      $width     = null,
      $height    = null)
    {
    	require_once(FH_INCLUDE_DIR.'fields/class.Editor.php');
    	require_once(FH_INCLUDE_DIR.'includes/fckeditor.php');


        // create a new editor
        $fld =& new Editor($this, $name);
        if(!is_null($validator)) $fld->setValidator($validator);
        if(!is_null($path))      $fld->setServerPath($path); 
        if(!empty($toolbar))     $fld->setToolbar($toolbar);
        if(!empty($skin))        $fld->setSkin($skin);
        if(!is_null($width))     $fld->setWidth($width);
        if(!is_null($height))    $fld->setHeight($height);
        
        // add the editor to the fields array
        $this->_fields[$name] = array($title, &$fld);
    } 
    
    /**
     * FormHandler::dateField()
     *
     * Public: Create a dateField on the form
     *
     * @param string $title: The title of the field
     * @param string $name: The name of the field
     * @param string $validator: The validator which should be used to validate the value of the field
     * @param boolean $required: If the field is required to fill in or can the user leave it blank
     * @param string $display: How do we have to display the fields? These can be used: d, m and y.
     * @param string $interval: The interval between the current year and the years to start/stop.Default the years are beginning at 90 yeas from the current. It is also possible to have years in the future. This is done like this: "90:10" (10 years in the future).
     * @param string $extra: CSS, Javascript or other which are inserted into the HTML tag
     * @return void
     */
    function dateField(
      $title,
      $name,
      $validator = null,
      $required  = null,
      $display   = null,
      $interval  = null,      
      $extra     = null)
    {
    	require_once(FH_INCLUDE_DIR.'fields/class.SelectField.php');
    	require_once(FH_INCLUDE_DIR.'fields/class.DateField.php');

        // create a new datefield
        $fld =& new DateField($this, $name);
        if(!is_null($validator)) $fld->setValidator($validator);
        if(!is_null($required))  $fld->setRequired($required);
        if(!empty($display))     $fld->setDisplay($display);
        if(!is_null($interval))  $fld->setInterval($interval);        
        if(!is_null($extra))     $fld->setExtra($extra);        
        
        // add the datefield to the fields array
        $this->_fields[$name] = array($title, &$fld);
        
        $this->_date[] = $name;
    }  
    
    /**
     * FormHandler::jsDateField()
     *
     * Public: Create a dateField with a jscalendar popup on the form
     *
     * @param string $title: The title of the field
     * @param string $name: The name of the field
     * @param string $validator: The validator which should be used to validate the value of the field
     * @param boolean $required: If the field is required to fill in or can the user leave it blank
     * @param string $display: How do we have to display the fields? These can be used: d, m and y.
     * @param string $interval: The interval between the current year and the years to start/stop.Default the years are beginning at 90 yeas from the current. It is also possible to have years in the future. This is done like this: "90:10" (10 years in the future).
     * @param string $extra: CSS, Javascript or other which are inserted into the HTML tag
     * @return void
     */
    function jsDateField(
      $title,
      $name,
      $validator = null,
      $required  = null,
      $display   = null,
      $interval  = null,      
      $extra     = null)
    {
    	require_once(FH_INCLUDE_DIR.'fields/class.SelectField.php');
    	require_once(FH_INCLUDE_DIR.'fields/class.DateField.php');
    	require_once(FH_INCLUDE_DIR.'fields/class.jsDateField.php');
    	

        // create a new datefield
        $fld =& new jsDateField($this, $name);
        if(!is_null($validator)) $fld->setValidator($validator);
        if(!is_null($required))  $fld->setRequired($required);
        if(!empty($display))     $fld->setDisplay($display);
        if(!is_null($interval))  $fld->setInterval($interval);        
        if(!is_null($extra))     $fld->setExtra($extra);        
        
        // add the datefield to the fields array
        $this->_fields[$name] = array($title, &$fld);
        
        $this->_date[] = $name;
    }  
    
    /**
     * FormHandler::timeField()
     *
     * Public: Create a timeField on the form
     *
     * @param string $title: The title of the field
     * @param string $name: The name of the field
     * @param string $validator: The validator which should be used to validate the value of the field
     * @param int $format: 12 or 24. Which should we use?
     * @param string $extra: CSS, Javascript or other which are inserted into the HTML tag
     * @return void
     */
    function timeField(
      $title,
      $name,
      $validator = null,
      $required  = null,
      $format    = null,
      $extra     = null)
    {
    	require_once(FH_INCLUDE_DIR.'fields/class.SelectField.php');
    	require_once(FH_INCLUDE_DIR.'fields/class.TimeField.php');

        // create a new timefield
        $fld =& new TimeField($this, $name);
        if(!is_null($validator)) $fld->setValidator($validator);
        if(!is_null($required))  $fld->setRequired($required);
        if(!is_null($format))    $fld->setHourFormat($format); 
        if(!is_null($extra))     $fld->setExtra($extra);
        
        // add the timefield to the fields array
        $this->_fields[$name] = array($title, &$fld);
    }
      
    
    /*****************/
    /**** BUTTONS ****/
    /*****************/
    
    /**
     * FormHandler::button()
     *
     * Public: Create a button on the form
     *
     * @param string $caption: The caption of the button
     * @param string $name: The name of the button
     * @param string $extra: CSS, Javascript or other which are inserted into the HTML tag
     * @return void
     */
    function button( $caption, $name = null, $extra = null) {
        // get new button name if none is given
        if(is_null($name)) {
            $name = $this->_getNewButtonName();
        }
             
        // create new submitbutton
        $btn =& new Button( $this, $name );        
        $btn->setCaption( $caption );
        if(!is_null($extra)) $btn->setExtra($extra);
        
        // add the button to the fields array
        $this->_fields[$name] = array('__BUTTON__', &$btn);
    }
    
    /**
     * FormHandler::submitButton()
     *
     * Public: Create a submitButton on the form
     *
     * @param string $caption: The caption of the button
     * @param string $name: The name of the button
     * @param string $extra: CSS, Javascript or other which are inserted into the HTML tag
     * @param boolean $disableOnSubmit: Disable the button when it is pressed
     * @return void
     */
    function submitButton( $caption = null, $name = null, $extra = null, $disableOnSubmit = null) {
        require_once(FH_INCLUDE_DIR.'buttons/class.SubmitButton.php');

    	// get new button name if none is given
        if(is_null($name)) {
            $name = $this->_getNewButtonName();
        }
         
        // create new submitbutton
        $btn =& new SubmitButton($this, $name);
        if(!is_null($caption))         $btn->setCaption($caption);         
        
        if(!is_null($extra))           $btn->setExtra($extra);
        if(!is_null($disableOnSubmit)) $btn->disableOnSubmit( $disableOnSubmit );
        
        // add the button to the fields array
        $this->_fields[$name] = array('__BUTTON__', &$btn);
    }
    
    /**
     * FormHandler::imageButton()
     *
     * Public: Create a imageButton on the form
     *
     * @param string $image: The image URL which should be a button
     * @param string $name: The name of the button
     * @param string $extra: CSS, Javascript or other which are inserted into the HTML tag
     * @param boolean $disableOnSubmit: Disable the button when it is pressed
     * @return void
     */
    function imageButton( $image, $name = null, $extra = null, $disableOnSubmit = null) {
        require_once(FH_INCLUDE_DIR.'buttons/class.ImageButton.php');

    	// get new button name if none is given
        if(is_null($name)) {
            $name = $this->_getNewButtonName();
        }
         
        // create the image button
        $btn =& new ImageButton( $this, $name, $image );
        if(!is_null($extra))           $btn->setExtra( $extra );
        if(!is_null($disableOnSubmit)) $btn->disableOnSubmit( $disableOnSubmit );
        
        // add the button on the form
        $this->_fields[$name] = array('__BUTTON__', &$btn);
    }  
   
    /**
     * FormHandler::resetButton()
     *
     * Public: Create a resetButton on the form
     *
     * @param string $caption: The caption of the button
     * @param string $name: The name of the button
     * @param string $extra: CSS, Javascript or other which are inserted into the HTML tag
     * @return void
     */
    function resetButton($caption = null, $name = null, $extra = null) {        
        require_once(FH_INCLUDE_DIR.'buttons/class.ResetButton.php');

    	// get new button name if none given
        if(is_null($name)) {
            $name = $this->_getNewButtonName();
        }
         
        // create new resetbutton
        $btn =& new ResetButton($this, $name);        
        if(!is_null($caption)) $btn->setCaption($caption);
        if(!is_null($extra))   $btn->setExtra($extra);
        
        // add the button to the fields array
        $this->_fields[$name] = array('__BUTTON__', &$btn);
    } 
    
    /**
     * FormHandler::cancelButton()
     *
     * Public: Create a cancelButton on the form
     *
     * @param string $caption: The caption of the button
     * @param string $url: The URL to go to when the button is clicked
     * @param string $name: The name of the button
     * @param string $extra: CSS, Javascript or other which are inserted into the HTML tag
     * @return void
     */
    function cancelButton($caption = null, $url = 'history.back(-1)', $name = null, $extra = null) {
        // get new button name if none given
        if(is_null($name)) {
            $name = $this->_getNewButtonName();
        }
        
        // where to go when the button is clicked...
        if(ereg('history', $url)) {
            $extra .= ' onclick="'.$url.'"';
        } else {
            $extra .= ' onclick="document.location.href=\''.$url.'\'"';
        }
        
        // if no caption is given, get our own caption
        if(is_null($caption)) {
            $caption = $this->_text( 28 );
        }
        
        // create new button
        $btn =& new Button($this, $name);        
        $btn->setCaption($caption);
        if(!is_null($extra))   $btn->setExtra($extra);
        
        // add the button to the fields array
        $this->_fields[$name] = array('__BUTTON__', &$btn);
    }
    
    /**
     * FormHandler::backButton()
     *
     * Generate a back button to go one page back in a multi-paged form
     *
     * @param string $caption: The caption of the button
     * @param string $name: The name of the button
     * @param string $extra: CSS, Javascript or other which are inserted into the HTML tag
     * @return void
     */
    function backButton( $caption = null, $name = null, $extra = null) {
    	static $setJS = false;
    	
    	// include the needed javascript file
    	if( !$setJS ) {
    		$this->_setJS(FH_FHTML_DIR.'js/page_back.js', true);
    		$setJS = true;
    	}
    	
    	// get new button name if none given
        if(is_null($name)) {
            $name = $this->_getNewButtonName();
        }
        
        $extra = 'onclick="pageBack(document.forms[\''.$this->_name.'\']);"';
        
        // if no caption is given, get our own caption
        if(is_null($caption)) {
            $caption = $this->_text( 28 );
        }
        
        // create new button
        $btn =& new Button($this, $name);        
        $btn->setCaption($caption);
        if(!is_null($extra))   $btn->setExtra($extra);
        
        // add the button to the fields array
        $this->_fields[$name] = array('__BUTTON__', &$btn);
    	
    } 
    
    /*******************/
    /****** STYLE ******/
    /*******************/
    
    /** 
	 * FormHandler::enableViewMode() 
	 * 
	 * @param boolean $enabled: The new state of the Forms View Mode 
	 * @return void 
	 */      
	function enableViewMode($enable = true) {  
	    $this->_viewMode = $enable; 
	} 

	/** 
     * FormHandler::setMaxLength() 
     * 
     * Public: set the maximum length of a TextArea 
     * 
     * @param string $field: The field for which the maximum length will be set 
     * @param int $maxlength: The allowed max input length of the field 
     * @param boolean $displaymessage: determines if a message is displayed with characters left 
     * @return void 
     */ 
    function setMaxLength( $field, $maxlength, $displaymessage = true ) { 
        static $setJSmaxlength = false; 

        // check if the field exists and is correct 
        if( !$this->fieldExists($field) || strtolower(get_class( $this->_fields[$field][1] )) != 'textarea') { 
            trigger_error( 'You have to declare the textarea first! The field "'.$field.'" does not exists in the form!', E_USER_WARNING ); 
            return; 
        } 

        // check if the maxlength is numeric 
        if( !is_numeric( $maxlength ) ) { 
            trigger_error( 'You have to give an numeric maxlength!', E_USER_WARNING ); 
            return; 
        } 

        // add the javascript file if not done yet 
        if( !$setJSmaxlength ) { 
            $setJSmaxlength = true; 
            $this->_setJS(FH_FHTML_DIR.'js/maxlength.js', true); 
        } 

        // add the javascript to the fields "extra" argument 
        $this->_fields[$field][1]->setExtraAfter( 
          "<br /><script language='javascript' type='text/javascript'>\n".
          "displaylimit('".$this->_name."','".$field."','".$maxlength."',0".$displaymessage.",'".$this->_text(36) ."');\n".
          "</script>\n"
        );
    } 

    
    /**
     * FormHandler::setHelpText()
     *
     * Public: set the help text for a specific field
     *
     * @param string $field: The name of the field to set the help text for
     * @param string $helpText: The help text for the field
     * @param string $helpTitle: The help title 
     * @return void
     */
    function setHelpText( $field, $helpText, $helpTitle = null ) { 
    	static $setJS = false;
    	// make sure that the overlib js file is included
    	if(!$setJS) {
    		$setJS = true;
    		$this->_setJS(FH_FHTML_DIR.'overlib/overlib.js', true);
    		$this->_setJS(FH_FHTML_DIR.'overlib/overlib_hideform.js', true);

    	}
    	
    	$helpTitle = is_null($helpTitle) ? "%title% - Help" : htmlentities($helpTitle);
    	
    	$helpTitle = preg_replace("/\r?\n/", "\\n", addslashes( $helpTitle ));
    	$helpText  = preg_replace("/\r?\n/", "\\n", addslashes( $helpText ));
    	
    	$this->_help[$field] = array(
    	  htmlentities($helpText),
    	  $helpTitle
    	);  
    }
    
    /**
     * FormHandler::setHelpIcon()
     *
     * Public: set the help icon used for help messages
     *
     * @param string $helpIcon: The path to the help icon
     * @return void
     */
    function setHelpIcon( $helpIcon ) { 
    	$this->_helpIcon = $helpIcon;
    }
    
    /**
     * FormHandler::addHTML()
     *
     * Public: Add some HTML to the form
     *
     * @param string $html: The HTML we have to add to the form
     * @return void
     */
    function addHTML( $html ) {
    	static $iCounter = 1;
        $this->_fields['html'.$iCounter++] = array('__HTML__', $html);
    }
    
    /**
     * FormHandler::addLine()
     *
     * Public: Add a new row to the form. 
     *
     * @param string $data: Possible data to set into the row (line)
     * @return void
     */
    function addLine( $text = null ) {
    	static $iCounter = 1;
        $this->_fields['line'.$iCounter++] = array('__LINE__', 
          sprintf( FH_LINE_MASK, $text )
        );
    }  
    
    /**
     * FormHandler::borderStart()
     *
     * Public: Begin a new fieldset 
     *
     * @param string $caption: The caption of the fieldset
     * @param string $name: The name of the fieldset
     * @return void
     */
    function borderStart( $caption = null, $name = null ) {
        static $i = 1;
        
        if( is_null( $name ) ) {
		    $name = 'fieldset'.$i++;
        }
        
        $this->_fields[] = array('__FIELDSET__', array($name, $caption));
    }
    
	/**
     * FormHandler::borderStop()
     *
     * Public: Stops a fieldset 
     *
     * @return void
     */
    function borderStop() {
        $this->_fields[] = array('__FIELDSET-END__', true);
    }
    
    /**
     * FormHandler::setMask()
     *
     * Public: Sets a mask for the new row of fields 
     *
     * @param string $mask: The mask we have to use
     * @param int|bool $repeat: If we have to repeat the mask. When a integer is given, it will be countdown
     * @return void
     */
    function setMask( $mask = null, $repeat = true, $setTable = null ) {
		if(is_null($mask)) {
			$mask = FH_DEFAULT_ROW_MASK;
		} else if(file_exists($mask) && is_file($mask)) { // double check of PHP bug in file_exists
			if(is_readable($mask)) {
				if( function_exists('file_get_contents')) {
            		$mask = file_get_contents( $mask );
				} else {
					$handle = fopen($mask, 'rb');
					$mask = '';
					while (!feof($handle)) {
					  $mask .= fread($handle, 4096);
					}
					fclose($handle);
				}
			} else {
				trigger_error('Could not read from template '.$mask, E_USER_WARNING );
			}			
        }
        
        // do we have to set a table arround the fields?
        if( !is_null( $setTable ) ) { 
        	$this->_setTable = $setTable;
        }
        
        $this->_fields[] = array( '__MASK__', array( $mask, $repeat ) );
    }
    
    /**
     * FormHandler::setErrorMessage()
     *
     * Public: Set a spicified error message to a field 
     *
     * @param string $field: The field to set the message for
     * @param string $message: The message to use when the fields value is invalid
     * @return void
     */
    function setErrorMessage( $field, $message, $useStyle = true ) {
    	$this->_customMsg[$field] = array( $message, $useStyle );
    }
    	
        
    /****************************/
    /**** DATA MANIPULATION *****/
    /****************************/
    
    /**
     * FormHandler::getValue()
     *
     * Public: alias for the function value
     *
     * @param string $field: The field which value we have to return
     * @return void
     */
    function getValue( $field ) { 
    	return $this->value( $field );
    }
    
    /**
     * FormHandler::value()
     *
     * Public: get the value of the requested field
     *
     * @param string $field: The field which value we have to return
     * @return void
     */
    function value( $field ) {
        if(!_global) global $_POST;
        
        // is it a field?
        if( isset( $this->_fields[$field] ) && is_object($this->_fields[$field][1]) && method_exists($this->_fields[$field][1], 'getvalue')  ) {
            return $this->_fields[$field][1]->getValue();
        // is it a field of the database ?
        } else if( isset( $this->_dbData ) && is_array( $this->_dbData ) && array_key_exists($field, $this->_dbData) ) {
            return $this->_dbData[$field];
        // is it a added value ?    
        } else if( isset($this->_add[$field]) ) {
            return $this->_add[$field];
        } else if( isset( $this->_cache[$field]) ) { 
        	return $this->_cache[$field];  
        } else if( isset( $this->_buffer[$field]) ) {
        	return $this->_buffer[$field];          
        // BELOW is beta!!    
        } else if( isset( $_POST[$field] ) ) {
            return $_POST[$field];
        /////    
        } else {
            trigger_error("Try to get the value of an unknown field '$field'!");  
            return null;
        }
    }
    
    /**
     * FormHandler::setValue()
     *
     * Public: set the value of the spicified field 
     *
     * @param string $field: The field which value we have to set
     * @param string $value: The value we have to set
     * @param boolean $overwriteCurrentValue: Do we have to overwrite the current value of the field (posted or db-loaded values)     
     * @return void
     */
    function setValue($sField, $sValue, $bOverwriteCurrentValue = false) {
    	if( $this->fieldExists( $sField ) ) {
    		// if the field does not exists in the database 
    	    if($bOverwriteCurrentValue || (!isset($this->_dbData[$sField]) && !$this->isPosted() )) {
    	    	$this->_fields[$sField][1]->setValue( $sValue );
    	    }
    	} else {    		       
    		// save the data untill the field exists
    		$this->_buffer[$sField] = array($bOverwriteCurrentValue, $sValue);	                    
    	}
    }
    
    /**
     * FormHandler::addValue()
     *
     * Public: add a value to the data array which is going 
     * to be saved/used in the oncorrect & onsaved functions 
     *
     * @param string $field: The field which value we have to set
     * @param string $value: The value we have to set
     * @param boolean $sqlFunction: Is the value an SQL function ?
     * @return void
     */ 
    function addValue($field, $value, $sqlFunction = false) {
        $this->_add[$field] = $value;                  
        
        // add to the sql list if the value is a sql function
        if($sqlFunction) {
            $this->_sql[] = $field;
        }
    }
    
    /**
     * FormHandler::setAutoComplete()
     *
     * Public: set a list of items for auto complete
     *
     * @param string $field: The field which should be auto complete
     * @param array $options: The list of options for the uto complete
     * @return void
     */
    function setAutoComplete( $field, $options ) { 
    	static $setJS = false;
    	
    	// check if the field exists and is correct
    	if( !$this->fieldExists($field) || strtolower(get_class( $this->_fields[$field][1] )) != 'textfield') { 
    		trigger_error( 'You have to declare the textfield first! The field "'.$field.'" does not exists in the form!', E_USER_WARNING );
    		return;    		
    	}
    	
    	// check if the options are correct
    	if( !is_array( $options ) ) {
    		trigger_error( 'You have to give an array as options!', E_USER_WARNING );
    		return;
    	} 
    	
    	// add the javascript file if not done yet
    	if( !$setJS ) { 
    		$setJS = true;	
    		$this->_setJS(FH_FHTML_DIR.'js/autocomplete.js', true);
    	}
    	
    	// create the javascript array
    	$js = $field.'_values = [';
		foreach( $options as $option ) { 
			$js .= '"'.str_replace('"', '\"', $option) .'", ';
    	}
    	$this->_setJS( substr($js, 0, -2)."];\n" );    	
    	
    	// add the javascript to the fields "extra" argument
    	$this->_fields[$field][1]->_sExtra .= "onkeypress='return autocomplete(this, event, ".$field."_values);'";    		
    }
    
    /***********************/
    /** DATABASE OPTIONS ***/
    /***********************/
    
    /**
     * FormHandler::dbInfo()
     *
     * Public: set the DBInfo. You can give the primary key's after the table name, like this:
     *
     * dbInfo( 'DB', table', 'mysql', 'prKey1');
     *
     * or
     *
     * dbInfo( 'DB', table', 'mysql', 'prKey1', 'prKey2', 'etc...');
     *
     * When you dont specify a primary key, FormHandler will try to 
     * retrieve it automaticly. When this failes, it will trigger an error.
     *
     * @param string $db: the db which we are using
     * @param string $table: the table where we have to save the data
     * @param string $type: the type of db which is used
     * @return
     */
    function dbInfo( $db, $table, $type = null ) {
    	require_once( FH_YADAL_DIR . 'class.Yadal.php' );       
    	
    	// if no db-type is given, set the default
    	if(is_null($type)) {
    	   $type = FH_DEFAULT_DB_TYPE;
    	}
    	
    	// get prKey(s) if given
        /*
        $numArgs = func_num_args();
        if($numArgs > 3) {
            $keys = array();
            for($i = 3; $i < $numArgs; $i ++ ) {
                $keys[] = func_get_arg($i);
            }                
        } else {
            $keys = null;
        }
        */
        
        $this->_table = $table;
        
        $this->_db = newYadal( $db, $type );
        
        // try to fetch the database data allready
        if( $this->edit && $this->_db->isConnected() ) {
            $this->_loadDbData();
        }
    }
    
    /**
     * FormHandler::dbConnect()
     *
     * Public: Connect to the database (only needed when there is no connection yet!)
     *
     * @param string $username: the username used to login
     * @param string $password: the password used to login
     * @return void
     */
    function dbConnect($host = null, $username = '', $password = '') {
        if( is_object( $this->_db ) ) {
            if(is_null($host)) {
                $host = FH_DEFAULT_DB_HOST;
            }
            
            if( !$this->_db->connect( $host, $username, $password ) ) {
            	trigger_error( 
            	  'Error, connecting failed: '.$this->_db->getError(), 
            	  E_USER_WARNING 
            	);
            	return false;
            }
        } else {
            trigger_error(
              'No database object available! Make sure you call DBInfo() first!', 
              E_USER_WARNING
            );
            $this->_db = null;
            return;            
        }
        
        // load the database values only on a edit form...
        if( $this->edit && ( !is_array($this->_dbData) || sizeof($this->_dbData) == 0) ) {
            $this->_loadDbData();
        }
    }  
    
    /**
     * FormHandler::setConnectionResource()
     *
     * <description>
     *
     * @param string $table: The table which should be used to save the data in 
     * @param resource $conn: 
     * @return void
     * @access public
     * @author Teye Heimans   
     */
    function setConnectionResource( $conn, $table = null, $type = null ) {
    	require_once( FH_YADAL_DIR.'class.Yadal.php' );
    	
    	// if no db-type is given, set the default
    	if(is_null($type)) {
    	   $type = FH_DEFAULT_DB_TYPE;
    	}
    	
    	// make this function backwards compatible ( $table, $conn, $type  as arguments )
    	if( is_string( $conn ) && is_resource( $table ) ) {
    	    $tmp   =& $table;
    	    unset( $table );
    	    $table =& $conn;
    	    unset( $conn );
    	    $conn  =& $tmp;
    	    unset( $tmp );
    	}
    	
    	// save the table name
    	$this->_table = $table;
    	
    	// create the yadal object
    	$this->_db = newYadal(null, $type); 
    	
    	// set the sql resource which we should use
    	$this->_db->setConnectionResource( $conn ); 
    	
    	// try to fetch the database data allready
        if( $this->edit ) {
            $this->_loadDbData();
        }   	
    }
    
    
    /*******************/
    /***** GENERAL *****/
    /*******************/
    
    /**
     * FormHandler::getPage()
     *
     * Returns the page number of the current form (used when newPage is used!)
     *
     * @return int
     * @access public
     * @author Teye Heimans   
     */
    function getPage() { 
    	return $this->_pageCounter;
    }
    /**
     * FormHandler::linkSelectFields()
     *
     * Public: link de given selectfields (load the values dynamicly)
     *
     * @param string $filename: the name of the file which will load the new values for the select field
     * @param string $fields: the name of the first dynamic select field.
     * @param ...: More fields which are linked to eachother    
     * @return null
     */
    function linkSelectFields( $filename, $fields )  {
    	static $setJS = false;
    	
    	// add the javascript file if not done yet
    	if( !$setJS ) { 
    		$setJS = true;	
    		$this->_setJS(FH_FHTML_DIR.'js/linked_select.js', true);
    	}
    	 
        $js='';         
        for( $i = 1; $i < (func_num_args()-1); $i++) { 
            $fld1 = func_get_arg($i); 
            $fld2 = func_get_arg($i+1); 
            
            if( $i == 1) {
            	if( !isset($this->_dynamicLoader)) {
            		$this->_dynamicLoader = '';
            	}
            	$this->_dynamicLoader .= 
            	"// load the first item of the dynamic select fields\n".
            	"attach".$fld1."();\n\n";
            }

            $func  = $i < ( func_num_args() - 2 ) ? 'attach'.$fld2 : 'null';             
            $param = $this->edit ? '&value=' . $this->value($fld2) : ''; 

            $js.=
            'function attach'.$fld1.'() {'."\n".
			'    attachelement("'.$fld1.'", "change", load'.$fld2.');'."\n". 
            '    load'.$fld2.'();'."\n".
            '}'."\n\n".
			'function load'.$fld2.'() {'."\n". 
            '    value = GetElement("'.$fld1.'").value;'."\n". 
            //'    //GetElement("'.$fld2.'").innerHTML = "loading";'."\n". 
            '    loadexternal('."\n".
            '      "'.$filename.'",'."\n". 
            '      "linkselect=true&filter="+value+"&field='.$fld2.$param.'",'."\n". 
            '      "'.$fld2.'",'."\n". //display
            '      '.$func."\n".
            '    )'."\n". 
            '}'."\n";
        } 
        
        $this->_setJS($js, false); 
        
    } 
    
    /**
     * FormHandler::setDynamicOptions()
     *
     * Public static: make a javascript array of the given php array. This is
     * used for dynamic select fields
     *
     * @param array $array: the new options for the select field
     * @return null
     */
    function setDynamicOptions( $array, $useArrayKeyAsValue = true ) { 
		$output = 'var options = Array('." \n";
		
		foreach( $array as $key => $value ) { 
			$key = $useArrayKeyAsValue ? $key : $value;
			$output .= '  Array("'.htmlspecialchars($key).'", "'.htmlspecialchars($value).'"),'."\n";
		}	
		
		$output = substr($output, 0, -2);
		$output .= "\n);\n";
		
		echo $output;
	}
    
    /**
     * FormHandler::getTitle()
     *
     * Public: return the title of the given field name
     *
     * @param string $sField: The fieldname where to retrieve the title from
     * @return string
     */
    function getTitle( $sField ) {		
    	if(isset($this->_fields[$sField]) && is_object( $this->_fields[$sField][1] )) {    		
    		$sClass = strtolower( get_class( $this->_fields[$sField][1] ) );
    		  
    		if( is_subclass_of( $this->_fields[$sField][1], 'field') ) {
    			return $this->_fields[$sField][0];
    		} else {
    			trigger_error(
    			  'Error, cannot retrieve title of this kind of field: '.$sClass, 
    			  E_USER_WARNING 
    			);
    		}    			
    	} else {
    		trigger_error(
    		  'Could not find field "'.$sField.'"',
    		  E_USER_WARNING
    		);  
    	}
    	return null;
    }   
    
    /**
     * FormHandler::getLanguage()
     *
     * Public: return the language used for the form
     *
     * @return string: the language
     */
    function getLanguage() { 
    	return $this->_lang;
    }
    
    /** 
	 * FormHandler::isViewMode() 
	 * 
	 * Public: gets the ViewMode state 
	 * 
	 * @return boolean 
	 */      
	function isViewMode() {  
	    return $this->_viewMode; 
	}    

    /**
     * FormHandler::fieldExists()
     *
     * Public: check if the field exists in the form
     *
     * @param string $sField: The field to check if it exists in the form or not
     * @return boolean
     */
    function fieldExists( $sField ) {
    	return array_key_exists( $sField, $this->_fields);
    } 
    
    /**
     * FormHandler::getFormName()
     *
     * Public: return the name of the form
     *
     * @return string: the name of the form
     */
    function getFormName() {
    	return $this->_name;
    }
    
  	/**
     * FormHandler::getJavascriptCode()
     *
     * Public: return the needed javascript code for this form
     *
     * @return string: the needed javascript code for this form
     */
    function getJavascriptCode() {
    	static $return = false;
    	
    	if( !$return ) {
    		$result = ''; 
    		if( isset( $this->_js['file'] ) && is_array($this->_js['file'])) { 
	    		foreach( $this->_js['file'] as $line ) { 
	    			$result .= '<script language="javascript" type="text/javascript" src="'.$line.'"></script>'."\n";
	    		}
    		}
    		if( isset($this->_js['code']) && is_array($this->_js['code']) ) {
    			$result .= '<script language="javascript" type="text/javascript">'."\n";
    			foreach( $this->_js['code'] as $code ) {
    				$result .= $code;
    			}
    			$result .= "</script>\n"; 
    		}
    		$return = true;
    		return $result;
    	}   		 
    	return '';
    }
    
    /**
     * FormHandler::mailForm()
     *
     * Public: send the values of the form to the mail adress
     *
     * @param string $to: The e-mail adres to send the data to
     * @param string $subject: The subject of the mail 
     * @param string $headers: The headers which should be used by sending the mail
     * @return void
     */
    function mailForm( $to, $subject = null, $headers = null ) {
    	require_once(FH_INCLUDE_DIR.'includes/class.fhMailer.php');
    	
        if(is_null($subject)) {
            $subject = "New form mailed to you by FormHandler";
        }
        if(is_null($headers)) {
            $headers = "From: FormHandler <info@formhandler.nl>";
        }
        
        // add the mail items 
        $this->_mail[] = array( $to, $subject, $headers );
    }     
    
    /**
     * FormHandler::newPage()
     *
     * Public: Put the following fields on a seperate page
     *
     * @return void
     */
    function newPage() {
        $this->_fields[] = array( '__PAGE__', $this->_pageCounter++);
    }
    
    /**
     * FormHandler::setTabIndex()
     *
     * Public: set the tab index for the fields
     *
     * @param mixed $mTabs: array of comma seperated string with the field names.
     * When an array is given the array index will set as tabindex
     * @return void
     */
    function setTabIndex( $mTabs ) {
        if(is_string( $mTabs ) ) {
            $aTabs = explode(',', $mTabs);
            array_unshift($aTabs, '');
        } else if( is_array( $mTabs )) {
            // is set element 0, then move all elements 
            // (0 is not a valid tabindex, it starts with 1)
            if( isset( $mTabs[0])) {                
                ksort( $mTabs );
                $aTabs = array();
                
                foreach( $mTabs as $key => $value ) {
                    while( array_key_exists( $key, $aTabs) || $key <= 0) $key++;
                    $aTabs[$key] = $value;                    
                }
            } else {
                $aTabs = $mTabs;
            }
        }
        
        // array with tabs set ?
        if( isset( $aTabs ) ) { 
            foreach($aTabs as $key => $value ) {
                if(!empty($value)) {
                    $aTabs[$key] = trim($value);
                } else {
                    unset($aTabs);
                }
            }
            $this->_tabindexes = $this->_tabindexes + $aTabs ;
        }
    }

    /**
     * FormHandler::setLanguage()
     *
     * Public: Set the language we should use for error messages etc.
     * If no language is given, try to get the language defined by the visitors browser. 
     *
     * @param string $language: The language we should use
     * @return void
     */
    function setLanguage( $sLanguage = null ) {
        if(!_global) global $_SERVER;
        
    	if(is_null($sLanguage)) {
    		
    		// auto detect language ?
    		$bSet = false;
    		if( FH_AUTO_DETECT_LANGUAGE ) {
	    		// get all accepted languages by the browser
		        $aLang = array();
		        if(isset($_SERVER['HTTP_ACCEPT_LANGUAGE'])) {
		        	foreach(explode(',', $_SERVER['HTTP_ACCEPT_LANGUAGE']) as $sValue) {
		            	if(strpos($sValue, ';') !== false) {
		                	list($sValue, ) = split(';', $sValue);
		            	}
			            if(strpos($sValue, '-') !== false) {
			                list($sValue, ) = split('-', $sValue);
			            }
			            $aLang[] = $sValue;
			        }
		        }
		        
		        // set the language which we have too
		        while (list (, $l) = each ($aLang)) {
		        	if(file_exists(FH_INCLUDE_DIR.'language/'.strtolower($l).'.php')) {
		        		$this->setLanguage( $l );
		        		$bSet = true;
		        		break;
		        	}
		        }
    		}	
		       
    		// no language is set yet.. set the default language 
			if(!$bSet) {
				$this->setLanguage( FH_DEFAULT_LANGUAGE );
			}    		
    	} else {    	
	        $this->_lang = strtolower($sLanguage);	        
	        	        
	        if(file_exists(FH_INCLUDE_DIR.'language/'.$this->_lang.'.php')) {
	            include(FH_INCLUDE_DIR.'language/'.$this->_lang.'.php');
	            // load the array from the text file
	            $this->_text =& $fh_lang;
	        } else {
	            trigger_error('Unknown language: '.$this->_lang.'. Could not find file '.FH_INCLUDE_DIR.'language/'.$this->_lang.'.php!', E_USER_ERROR);
	        }
    	}
    }
    
    /**
     * FormHandler::catchErrors()
     *
     * Public: get the errors occoured in the form
     *
     * @param boolean $display: If we still have to display the errors in the form (default this is disabled)
     * @return array of errors or an empty array if none occoured
     */
    function catchErrors( $display = false ) {
        
        // only return the errors when the form is posted!
        if($this->isPosted()) {
            $this->_displayErrors = $display;
            
            $errors = array();
 
            foreach( $this->_fields as $field => $obj ) {
            	$obj = $this->_fields[$field];
                if(method_exists($obj[1], 'getError') && method_exists($obj[1], 'isValid')) {
                    $obj[1]->isValid();
                    $err = $obj[1]->getError();
                    if(strlen($err) > 0) {
                        $errors[$field] = $err;
                    }
                }
            }
            return $errors;
        } 
        return array();
    }
    
    /**
     * FormHandler::resizeImage()
     *
     * Public: resize the image uploaded in the given field
     *
     * @param string $field: The field where the image is uploaded
     * @param string $saveAs: How the image has to be saved (if not given, the original wil be overwritten)
     * @param int $maxWidth: The maximum width of the resized image
     * @param int $maxHeight: the maximum height of the resized image
     * @param int $quality: the quality of the resized image     
     * @return void
     */
    function resizeImage( $field, $saveAs = null, $maxWidth = null, $maxHeight = null, $quality = null )  {
    	require_once(FH_INCLUDE_DIR.'includes/class.imageconverter.php');
    	
        // is gd enabled ?
        if( ! ImageConverter::GDVersion() ) {
            trigger_error( 'Error! To use the function resizeImage you have to enable GD Libary!', E_USER_WARNING);
            return;
        }
                
        if(is_null($maxWidth))  $maxWidth  = FH_DEFAULT_RESIZE_WIDTH;
        if(is_null($maxHeight)) $maxHeight = $maxWidth;

        $this->_convert[$field]['resize'][] = array( $saveAs, $maxWidth, $maxHeight, $quality );        
    }
     
    /**
     * FormHandler::mergeImage()
     *
     * Public: merge a image uploaded in the given field with another image
     *
     * @param string $field: The field where the image is uploaded
     * @param string $merge: The image which we should merge
     * @param int $align: The align of the merge image (eg: left, center, right)
     * @param int $valign: The vertical align of the merge image( eg: top, middle, bottom)   
     * @return void
     */
    function mergeImage( $field, $merge, $align = 'center', $valign = 'bottom', $transparantColor = null ) {
        require_once(FH_INCLUDE_DIR.'includes/class.imageconverter.php');
    	
        // is gd enabled ?
        if( ! ImageConverter::GDVersion() ) {
            trigger_error( 'Error! To use the function resizeImage you have to enable GD Libary!', E_USER_WARNING);
            return;
        }
        
        $this->_convert[$field]['merge'][] = array( $merge, $align, $valign, $transparantColor );
    }
    
    /**
     * FormHandler::setFocus()
     *
     * Public: set the focus to a sepcific field
     *
     * @param string $field: The field which should get the focus
     * @return boolean: true if the focus could be set, false if not
     */
    function setFocus($field) {
    	// if the field is false, no focus has to be set...
    	if( $field === false ) {
    		$this->_focus = false;
    		return true;
    	}
    	
        if(!isset($this->_fields[$field])) {
            trigger_error('Could net set focus to unknown field '.$field);
            return;
        } 
        
        // some fields have gotten other names... change it.
        switch ( get_class($this->_fields[$field][1])) {
            case 'datefield':
              $field = $field.'_day';
              break;
            case 'listfield':
              $field = $field.'_ListOn';
              break;
            case 'timefield':
              $field = $field.'_hour';
              break;
            case 'editor':
            case 'radiobutton':
            case 'checkbox':
            case 'hiddenfield':
            // buttons cant have the focus
            case 'submitbutton':
            case 'resetbutton':
            case 'imagebutton':
            case 'button':
              $field = null;
              break;  
        }
        
        $this->_focus = $field;
        return !is_null($field);
    }
    
    /**
     * FormHandler::checkPassword()
     *
     * Public: Preform a password check on 2 password fields:
     * - both values are the same
     * - the values are longer then a minimum length (configured in the config file)
     * - on an add-form, the fields are required
     * - on an edit-form, the fields can be left empty, and the old password will stay (no changes will take place)
     *
     * @param string $field1: The first password field we should check
     * @param string $field2: The second password field we should check
     * @return void
     */
    function checkPassword($field1, $field2, $setEditMsg = true) {
        
        if(!isset($this->_fields[$field1]) || !isset($this->_fields[$field2]) || 
            strtolower( get_class( $this->_fields[$field1][1] ) ) != 'passfield' ||
            strtolower( get_class( $this->_fields[$field2][1] ) ) != 'passfield')
        {
            trigger_error('Error: unknown field used in checkPassword!');
            return;
        }
        
        // add some text to nitify the user that he only has to enter his 
        // password when he wants to change it
        if($this->edit && $setEditMsg) {
            $this->_fields[$field1][1]->setPre(
              $this->_text( 25 )
            );            
        } 
            
        // is the form posted
        if($this->isPosted()) {
            // let passfield 1 check if it matches passfield 2
            $this->_fields[$field1][1]->checkPassword( $this->_fields[$field2][1] );
        }
    }
    
    /**
     * FormHandler::onCorrect()
     *
     * Public: Set the function which has to be called when the form is correct
     *
     * @param string $callback: The name of the function
     * @return void
     */
    function onCorrect($callback) {
    	if(!is_array($callback)) {
    		if(function_exists($callback)) {
        	    $this->_onCorrect = $callback;
    		} else {
    			trigger_error("Error, the onCorrect function '".$callback."' does not exists!", E_USER_ERROR);
    		}
    	// check if the method exists		
    	} else {
    		if(method_exists($callback[0], $callback[1])) {
    			$this->_onCorrect =& $callback;
    		} else {
    			trigger_error(
    			  "Error, the onCorrect method '".$callback[1]."' does not exists ".
    			  "in object '".get_class($callback[0])."'!", 
    			  E_USER_ERROR);
    		}
    	}    			
    }
    
    /**
     * FormHandler::onSaved()
     *
     * Public: Set the function  which has to be called when the form data is saved in the database
     *
     * @param string $callback: The name of the function
     * @return void
     */
    function onSaved($callback) {
    	if(!is_array($callback)) {
    		if(function_exists($callback)) {
        	    $this->_onSaved = $callback;
    		} else {
    			trigger_error("Error, the onSaved function '".$callback."' does not exists!", E_USER_ERROR);
    		}
    	// check if the method exists		
    	} else {
    		if(method_exists($callback[0], $callback[1])) {
    			$this->_onSaved =& $callback;
    		} else {
    			trigger_error(
    			  "Error, the onSaved method '".$callback[1]."' does not exists ".
    			  "in object '".get_class($callback[0])."'!", 
    			  E_USER_ERROR);
    		}
    	}  
    }
    
    /**
     * FormHandler::isPosted()
     *
     * Public: If the form is posted
     *
     * @return boolean: if the form is posted or not
     */
    function isPosted() {
        return $this->_posted;
    }
    
    /**
     * FormHandler::isCorrect()
     *
     * Public: return if the form is filled correctly (for the fields which are set!)
     *
     * @return boolean: the form values valid or not
     */
    function isCorrect() {
    	// walk each field to check if they are valid
        $result = true;
        foreach( $this->_fields as $id => $data ) {        	
            // check if the fields are valid
            if(method_exists($this->_fields[$id][1], 'isvalid') && $this->_fields[$id][1]->isValid() !== true) {
                if(is_null($this->_focus)) {
                    $this->setFocus($id);
                }
                $result = false;
            
            // if multiple pages are used, only make sure that 
            // all pages untill the current page are correct    
            } else if($data[0] == '__PAGE__') { 
                if( $this->_curPage == $data[1]) {
                    break;
                }
            } 
        }
        
        // check if there is no unique field complication
        if( $result ) {
        	return $this->_checkUniqueFields();
        }
        
        return false;
    }
    
    /**
     * FormHandler::flush()
     *
     * Public: prints or returns the form
     *
     * @return string: the form or null when the form should be printed 
     */
    function flush( $return = false ) {    
        
        // when the form is not posted or the form is not valid
        if( !$this->_posted || !$this->isCorrect() ) {
        	// check if a value is set of an unknown field
        	if( sizeof( $this->_buffer ) > 0 ) {
        		foreach($this->_buffer as $sField => $a) {
        			trigger_error('Value set of unknown field '.$sField, E_USER_WARNING );
        		}        		
        	}
        
            // get the form
            $form = $this->_getForm();
            
        // when the form is not totaly completed yet (multiple pages)
        } else if( $this->_curPage < $this->_pageCounter ) {  
            // upload and convert uploads
            $this->_handleUploads();
                                  
            // get the next form
            $form = $this->_getForm( $this->_curPage + 1 );
            
        // when the form is valid
        } else {            
            // upload and convert uploads
            $this->_handleUploads();
            
            // generate the data array
            $data = array();
            foreach($this->_fields as $name => $fld) {
                if(method_exists($fld[1], 'getValue') && $name != $this->_name.'_submit') {
                    $data[$name] = $fld[1]->getValue();
                }
            }
            
            // add the user added data to the array
            $data = array_merge( $data, $this->_add );
            
            // do whe have to mail the form data?
            if( sizeof($this->_mail) ) {
                $this->_mailForm( $data );              
            }
            
            // call the users oncorrect function
            if(!empty($this->_onCorrect)) {
                if(is_array($this->_onCorrect)) {
                    $hideForm = call_user_method( $this->_onCorrect[1], $this->_onCorrect[0], $data);
                } else {
                    $hideForm = call_user_func( $this->_onCorrect, $data);
                }
            }
            
            // add the user added data again to the array (could have been changed!)
            $data = array_merge( $data, $this->_add );
            
            // if the db option is used
            if( !is_null($this->_db) ) {
                
            	// save the data into the databse
                $id = $this->_saveDbData( $data );
                
                if(is_array($id) && sizeof($id) == 1) { 
                	$id = $id[0];
                }
                
                // call the onsaved function
                if(!is_null($this->_onSaved)) {
                    if(is_array($this->_onSaved)) {
                        $hideForm = call_user_method( $this->_onSaved[1], $this->_onSaved[0], $id, $data);
                    } else {
                        $hideForm = call_user_func( $this->_onSaved, $id, $data);
                    }
                }
            }                
            
            // display the form again if wanted..
            if(isset($hideForm) && $hideForm === false) {
                $form = $this->_getForm();
            }  else if(is_string($hideForm)) {
                $form = $hideForm;
            } else {
                $form = '';
            }           
        }
        
        // cache all the fields values for the function value()
        foreach( $this->_fields as $fld => $value ) {
        	if( is_object($this->_fields[$fld][1]) && method_exists($this->_fields[$fld][1], "getvalue")) {
        		$this->_cache[ $fld ] = $this->_fields[$fld][1]->getValue();
        	}
        }
        
        // remove all vars to free memory
        foreach( get_object_vars($this) as $name => $value ) {
        	if( !in_array($name, array('_cache', 'edit', 'insert', '_posted' ) ) ) {
        		unset( $this->{$name} );
        	}
        }
        
        // disable our error handler!
        if( FH_DISPLAY_ERRORS ) {
        	restore_error_handler();
        }
        
        // return or print the form
        if( $return ) {
            return $form;
        } else {
            echo $form;
            return null;
        }
    }
}

?>