<?php

/**
* class Field
*
* Public abstract: class to create a field.
* This class contains code which is used by all the other fields
*
* Created by: T. Heimans
* Revision: $Date: 2005/10/02 19:48:23 $
*/
class Field {
    var $_oForm;         // object: the form where the field is located in
    var $_sName;         // string: name of the field
    var $_sValidator;    // string: callback function to validate the value of the field
    var $_mValue;        // mixed: the value of the field
    var $_sError;        // string: if the field is not valid, this var contains the error message
    var $_sExtra;        // string: extra data which should be added into the HTML tag (like CSS or JS) 
    var $_iTabIndex;     // int: tabindex or null when no tabindex is set
    var $_sExtraAfter;   // string: extra data which should be added AFTER the HTML tag 
    
    /**
     * Field::Field()
     *
     * Public abstract constructor: Create a new field 
     *
     * @param object $oForm: The form where the field is located on
     * @param string $sName: The name of the field     
     * @return void
     */
    function Field( &$oForm, $sName ) {
        $this->_oForm = &$oForm;  
        $this->_sName = $sName;
        
        // check if there are spaces in the fieldname
        if(strpos($sName,' ') !== false) {
            trigger_error('Warning: There are spaces in the field name "'.$sName.'"!', E_USER_WARNING );
        } 
        
        // get the value of the field
        if( $oForm->isPosted() ) {   
                     
            // make sure that the $_POST array is global
            if(!_global) global $_POST;
            
            // get the value if it exists in the $_POST array
            if( isset( $_POST[$sName] ) ) {
            	if( is_string( $_POST[$sName] ) ) { 
            		$this->setValue( 
	                  get_magic_quotes_gpc() ? stripslashes($_POST[$sName]) : $_POST[$sName]
	                );
            	} else if( is_array( $_POST[$sName] ) ) {
            		$item = array();
            		foreach ( $_POST[$sName] as $key => $value ) {
            			$item[$key] = get_magic_quotes_gpc() ? stripslashes($value) : $value;
            		}
            		$this->setValue($item);
            	}
            } 	
            // when the form is posted but this field is not found in the $_POST array,
            // keep the data from the db 
            // (This happens with the DISABLED attribute
            // Added at 02-09-05 by T. Heimans
            //} else if( $oForm->edit ) {
            //	if( isset( $oForm->_dbData[$sName] ) ) {
	        //        $this->setValue( $oForm->_dbData[$sName] );
	        //    }
            //} 
            	  
        // load database value if exists    
        } else if($oForm->edit) {
            if( isset( $oForm->_dbData[$sName] ) ) {
                $this->setValue( $oForm->_dbData[$sName] );
            }
        } 
        
        // check if the user got another value for this field.
        if( isset($oForm ->_buffer[ $sName ] ) ) {
        	
        	list( $bOverwrite, $sValue ) = $oForm->_buffer[ $sName ];
        	
        	// if the field does not exists in the database 
    	    if($bOverwrite || (!isset($oForm->_dbData[$sName]) && !$oForm->isPosted() )) {
    	    	$this->setValue( $sValue );
    	    }
    	    unset( $oForm->_buffer[ $sName ] ); 
        }
    }
    
    /**
     * Field::isValid()
     *
     * Public: Check if the value of the field is valid. If not,
     * set the error message and return false
     *
     * @return boolean: If the value of the field is valid
     */
    function isValid() {
    	// done this function before... return the prefious value
    	if( isset( $this->_isValid ) ) { 
    		return $this->_isValid;
    	}
        
        // is a validator set?
        if(isset($this->_sValidator) && $this->_sValidator != null) {
        	// if it's an array, it's a method 
            if (!is_array($this->_sValidator)) {
            	
	            // is the validator a user-spicified function?
	            if( function_exists($this->_sValidator) ) {
	                $error = call_user_func( $this->_sValidator, $this->getValue() );
	            } else {
	                $v =& new Validator();
	                if(method_exists($v, $this->_sValidator)) {
	                    // call the build in  validator function
	                    $error = $v->{$this->_sValidator}( $this->getValue() );
	                    
	                } else {
	                    trigger_error("Unknown validator: '".$this->_sValidator."' used in field '".$this->_sName."'");
	                    $error = false;
	                }
	                unset( $v );
	            }
            
			// method given
            } else {
            	// check if the method exists 
                if(method_exists($this->_sValidator[0], $this->_sValidator[1])) { 
                    $error = call_user_func(array($this->_sValidator[0], $this->_sValidator[1]), $this->getValue()); 
                } else { 
                    trigger_error( 
                      "Error, the validator method '".$this->_sValidator[1]."' does not exists ". 
                      "in object '".get_class($this->_sValidator[0])."'!", 
                      E_USER_ERROR
                    ); 
                    $error = false;
                }
            }
            
            // set the error message
            $this->_sError = 
              is_string($error) ? $error :
              (!$error ? $this->_oForm->_text( 14 ) :
			  (isset($this->_sError) ? $this->_sError : ''));            	
        } 
        
        $this->_isValid = empty( $this->_sError );
        return $this->_isValid;
    }
    
    /**
     * Field::setValidator()
     *
     * Public: set the validator which is used to validate the value of the field
     * This can also be an array. 
     * If you want to use a method to validate the value use it like this:
     * array(&$obj, 'NameOfTheMethod')
     *
     * @param string $sValidator: the name of the validator
     * @return
     */
    function setValidator( $sValidator ) {
        $this->_sValidator = $sValidator;
    }
    
    /**
     * Field::setTabIndex()
     *
     * Public: set the tabindex of the field
     *
     * @param int $iIndex
     * @return void
     */
    function setTabIndex( $iIndex ) {
        $this->_iTabIndex = $iIndex;
    }
    
    /** 
     * Field::setExtraAfter() 
     * 
     * Public: set some extra HTML, JS or something like that (to use after the html tag) 
     * 
     * @param strint $sExtra: the extra html to insert into the tag 
     * @return void 
     */ 
    function setExtraAfter( $sExtraAfter ) { 
        $this->_sExtraAfter = $sExtraAfter; 
    } 


    /**
     * Field::getValue()
     *
     * Public: return the value of the field
     *
     * @return mixed: the value of the field
     */
    function getValue() {
        return isset( $this->_mValue ) ? $this->_mValue : '';
    }
    
    /**
     * Field::getError()
     *
     * Public: return the error of the field (if the field-value is not valid)
     *
     * @return string: the error message
     */
    function getError() {
        return isset( $this->_sError) && strlen($this->_sError) > 0 ? sprintf( FH_ERROR_MASK ,$this->_sError): '';
    }
    
    /**
     * Field::setValue()
     *
     * Public: set the value of the field
     *
     * @param mixed $mValue: The new value for the field
     * @return void
     */
    function setValue( $mValue ) {
        $this->_mValue = $mValue; 
    }
    
    /**
     * Field::setExtra()
     *
     * Public: set some extra CSS, JS or something like that (to use in the html tag)
     *
     * @param strint $sExtra: the extra html to insert into the tag
     * @return void
     */
    function setExtra( $sExtra ) {
        $this->_sExtra = $sExtra;
    }
    
    /**
     * Field::getField()
     *
     * Public abstract: return the HTML of the field. 
     * This function HAS TO BE OVERWRITTEN by the child class!
     *
     * @return string: the html of the field
     */
    function getField() {
        trigger_error('Error, getField has not been overwritten!', E_USER_WARNING);
        return '';
    }
    
    /** 
	 * Field::getViewValue() 
	 * 
	 * Public: return the value of the field 
	 * 
	 * @return mixed: the value of the field 
	 */ 
	function getViewValue() { 
	    $val = $this->getValue(); 
	    
	    $return = '<input type="hidden" name="'.$this->_sName.'" value="'.$this->getValue().'" />';
	    
	    if( is_string($val) ) { 
	    	return $return . $val;
	    } else if( is_array($val) ) { 
	    	$return .= '<ul>';
	    	foreach($val as $item ) { 
	    		$return .= '<li>'.$item."</li>\n";
	    	}
	    	$return .= "</ul>\n";
	    	return $return;
	    } 
	   
	    return $return;
	} 

}

?>