<?php

/**
* class dbSelectField
*
* Create a select field from records retrieved from the db
*
* Created by: T. Heimans
* Revision: $Date: 2005/10/02 19:48:24 $
*/
class dbSelectField extends SelectField {
    var $_oDb;
	
    /**
     * dbSelectField::dbSelectField()
     *
     * Public constructor: create a new db selectfield object
     *
     * @param object &$oForm: the form where the datefield is located on
     * @param string $sName: the name of the datefield
     * @param object $oDb: object of the database handler
     * @param string $sTable: the table to get the fields from
     * @param mixed $mFields: array of string with the names of the fields which data we should get
     * @return
     */
	function dbSelectField( &$oForm, $sName, &$oDb, $sTable, $mFields, $sExtraSQL = null, $aMergeArray = array() ) {
	    // call the constructor of the selectfield
		$this->SelectField( $oForm, $sName );
		
		// make sure that the fields are set in an array
		$aFields = !is_array($mFields) ? array( $mFields ) : $mFields;
		
		$this->useArrayKeyAsValue( sizeof( $aFields) == 2 );
		
		// generate the query to retrieve the records
		$sQuery = 
		  'SELECT '. implode(', ', $aFields).
		  ' FROM '. $oDb->quote( $sTable).' '.$sExtraSQL;		  
		  
		// get the records and load the options
		$this->_aOptions = is_array($aMergeArray) ? $aMergeArray : array();  
		$sql = $oDb->query( $sQuery );  
		
		if( $sql ) 
		{
    		while( $row = $oDb->getRecord( $sql ) ) {
    			if( sizeof( $row ) == 1 ) {
    				$this->_aOptions[] = array_shift( $row );
    			} else {				
    		        $this->_aOptions[array_shift( $row )] = array_shift( $row );
    		    }
    		}
		} else {
		    trigger_error(
		      "Error, could not retrieve records.<br />\n".
		      "Error message: ". $oDb->getError()."<br />\n".
		      "Query: ". $sQuery,
		      E_USER_WARNING
		    );
		}  
 	}
}

?>