<?php


/**
 * class Handler
 *
 * handles all the private shyte of formhandler
 *
 */
class Handler {
	
	// private
    var $_fields;           // array: contains all the fields    
    var $_posted;           // boolean: if the form is posted or not
    var $_name;             // string: the name of the form
    var $_action;           // string: the action of the form
    var $_displayErrors;    // boolean: if we have to display the errors in the form
    var $_mask;             // string: the mask which should be used
    var $_upload;           // array: contains the names of the uploadfields
    var $_date;             // array: contains the names of the datefields
    var $_onCorrect;        // string: the callback function when the form is correct
    var $_onSaved;          // string: the callback function when the form is saved
    var $_add;              // array: contains the data which was added by the user
    var $_sql;              // array: contains the names of the added values which are sql functions
    var $_dontSave;         // array: dont save these fields    
    var $_focus;            // string: the field which should get the focus
    var $_convert;          // array: fields which should be converted (eg. resizeimage or mergeimage)
    var $_db;               // object: contains the database object if the option is used
    var $_id;               // array: the id(s) which we are editing
    var $_dbData;           // array: the database data 
    var $_buffer;           // array: buffer of set values (used when the field does not exists yet)
    var $_text;             // array: the language array we are using to display the messages etc   
    var $_lang;				// string: the language used
    var $_setTable;			// boolean: set a html table arround the fields or has the user done that in the mask ?
    var $_extra;			// string: extra tag information for the <form> tag (like CSS or javascript)
    var $_pageCounter;      // int: how many pages has this form
    var $_curPage;          // int: current page 
    var $_mail;             // array: contains the mailing data
    var $_tabindexes;       // array: tab indexes of the fields...
    var $_js;				// array: contains all the needed javascript for the form
    var $_help;				// array: contains the help text for the fields
    var $_helpIcon;		    // string: the path to the help image 
    var $_viewMode;			// boolean: is view mode enabled or not
    var $_cache;			// array: save the values of the field in this array after the flush is called (then the objects are deleted!)
    var $_table;			// string: the table name where we should save the data in
	
    /**
     * Handler::Handler()
     *
     * constructor: initialisation of some vars
     *
     * @return string: the new button name
     */
    function Handler() {
    	$this->_fields        = array();
        $this->_date          = array();
        $this->_upload        = array();
        $this->_add           = array();
        $this->_js		  	  = array();
        $this->_sql           = array();
        $this->_dbData        = array();
        $this->_dontSave      = array();
        $this->_buffer        = array();
        $this->_convert       = array();
        $this->_mail          = array();
        $this->_tabindexes    = array();
        $this->_customMsg     = array(); 
        $this->_help          = array();
        $this->_cache         = array();
        $this->_displayErrors = true;         
        $this->_setTable      = true;
        $this->_viewMode      = false;
        $this->_focus         = null;    
        $this->_pageCounter   = 1; 
    }
    
    /**
     * Handler::_getNewButtonName()
     *
     * Private: when no button name is given, get a unique button name
     *
     * @return string: the new button name
     */
    function _getNewButtonName() {
        static $counter = 1;
        
        return 'button'.$counter++;
    }
    
    /**
     * Handler::_setJS()
     *
     * Private: set the javascript needed for the fields
     *
     * @return void
     */
    function _setJS( $js, $isFile = false) { 
    	$this->_js[$isFile?'file':'code'][] = $js;
    }
    
    /**
     * Handler::_checkUniqueFields()
     *
     * Private: check if there are no double values for unique fields 
     *
     * @return boolean: false if the value is double
     */
    function _checkUniqueFields() {
    	// only check for unique fields if the rest of the field is correct and 
        // the database option is used... 
        if( isset($this->_db) && is_object($this->_db)) {
            $db =& $this->_db;              
            
            $unique  = $db->getUniqueFields ( $this->_table );
            $notnull = $db->getNotNullFields( $this->_table );
            
            // any unique fields found ?
            if(sizeof( $unique ) > 0) {
                
                // walk all unique fields
                foreach( $unique as $index => $fields) 
                {
                    $fieldInForm = false;
                    
                    $extra = array();
                    foreach( $fields as $field ) 
                    {
                        // does the unique field exists
                        if( $this->fieldExists( $field ) ) 
                        {
                            $fieldInForm = $field;
                            
                            // get the form-value of the field
                            $value = $db->escapeString( $this->_fields[$field][1]->getValue() );
                            
                            // quotes around the value ?
                            if( empty( $value ) && !in_array( $field, $notnull ) ) {
                                $value = 'NULL';
                            } else if(preg_match("/^-?([0-9]*\.?[0-9]+)$/", $value)) {
                                if( $db->_quoteNumbers ) {                         
                                    $value = "'".$value."'";
                                }
                            } else {
                                $value = "'".$value."'";
                            }
                        } 
                        // is the field a value which the user has added ?
                        else if( array_key_exists($field, $this->_add) ) 
                        {
                            $value = $this->_add[$field];
                            
                            if( !in_array( $field, $this->_sql) ) {
                                $value = "'".$db->escapeString( $value )."'";
                            }                                
                        }
                        else 
                        {
                            // field does not exists in the form, take a null value if
                            // possible, or else a empty string value
                            $value = in_array( $field, $notnull ) ? "''" : 'NULL';
                        } 

                        $extra[] = $field . ($value == 'NULL' ? ' IS NULL ' : ' = '.$value );                       
                    } 
                    
                    $where = $this->_getWhereClause( '!=', $extra );
                            
                    // check if the entry is double
                    $sql = $db->query(
                      'SELECT COUNT(1) AS num FROM '.$db->quote( $this->_table ). $where
                    );
                    
                    // if so, set the error message and return false;                        
                    if( $db->result( $sql, 0, 'num' ) > 0 && $fieldInForm) 
                    {
                        // get the error message
                        $error = 
                          sizeof($unique[$index]) == 1 ?
                          sprintf( 
                            $this->_text(35), 
                            $this->_fields[$fieldInForm][1]->getValue()
                          ) :
                          sprintf( 
                            $this->_text(39),
                            implode(', ', $unique[$index]),
                            $index
                          )  
                        ;
                        
                        // is there a field we can use to attach the error to?
                        if( $fieldInForm ) 
                        {
                            // set the error message
                            $this->_fields[$fieldInForm][1]->_sError = $error;
                        } 
                        // no field .. set as extra html
                        else 
                        {
                            $this->addLine($error);
                        }
                        
                        // return false (the form is not correct)
                        return false;
                    }        
                }                
            } 
        }
        return true; 
    } 
    
    /**
     * Handler::_mailForm()
     *
     * Private: mail de form data
     *
     * @return void
     */
    function _mailForm( $data ) { 
    	$mailer =& new fhMailer( $this->_mail );
                
        // add the attachements
        foreach( $this->_upload as $field ) {
            if( $this->_fields[$field][1]->isUploaded() ) {
                $mailer->attachFile(
                  $this->_fields[$field][1]->getSavePath().
                  $this->_fields[$field][1]->getValue()
                );
            }
        }
        // mail 
        $mailer->sendMail( $data );  
        unset( $mailer );
    }
    
    /**
     * Handler::_loadMask()
     *
     * Private: load the fields into the mask
     *
     * @param array $data: contains the title and the field object
     * @param string $mask: the mask which should be used
     * @param string $name: the name of the field
     * @return string: the filled mask, or null when the mask isn't filled completly
     */
    function _loadMask( $data, $mask, $name ) {
		static $result = null;
		
		$title =& $data[0];
		$obj   =& $data[1];
		
		if($title == '__BUTTON__') {
		    $title = '';
		} 

		// search and replace values
		$search = array(
		  '/%field%/ie',
		  '/%error%/ie',
		  '/%title%/i', 	
		  '/%seperator%/ie',
		  '/%name%/ie',
		  '/%value%/ie',
		  '/%help%/ie' 
		);


		$replace = array(
		  'str_replace("%", "____FH-percent____", '.
		  '$this->_viewMode && method_exists($obj, "getViewValue") ? $obj->getViewValue() : ((method_exists($obj, "getField")) ? $obj->getField() : '.
		  '((method_exists($obj, "getButton")) ? $obj->getButton() : "")))',
		  '($this->_displayErrors && method_exists($obj, "getError")) ? '.
		  '(array_key_exists($name, $this->_customMsg) && $obj->getError() != "" ? '.
		  '($this->_customMsg[$name][1] ? sprintf(FH_ERROR_MASK, $this->_customMsg[$name][0]) : $this->_customMsg[$name][0])'.
		  ' : $obj->getError()):""',
		  $title,
		  '!strlen($title)?"":":"',
		  '!empty($name)?$name:""',
		  'method_exists($obj, "getValue") ? (is_array($obj->getValue()) ? implode(", ", $obj->getValue()) : $obj->getValue()) : ""',
		  'array_key_exists($name, $this->_help) && (!isset($this->_viewMode) || !$this->_viewMode) ? sprintf(FH_HELP_MASK, $this->_helpIcon, $this->_help[$name][0], str_replace("%title%", htmlentities($title), $this->_help[$name][1])) : ""'
		);
		
		// for catching the errors
		if(!is_null($name) && method_exists($obj, 'getError')) {
		    $this->errors[$name] = $obj->getError();
		}

		// return a possible half filled mask if requested
		if( is_null($obj) && is_null($mask))
		    return preg_replace ( $search, array_fill(0, 4, ''), $result );

		// is the last mask finished? (if not, load it)
		if( !is_null($result) ) {
		    $mask = $result;
		}
		
		// replace the values
        $mask = preg_replace ( $search, $replace, $mask, 1 );

        // is the mask completly filled?
        $matches = array();
        preg_match_all('/(%field%|%error%|%title%|%seperator%|%name%|%value%)/i', $mask, $matches);
        if( isset($matches[1]) && count($matches[1]) > 0) {			
			$result = $mask;
			return null;
		} else {
			$result = null;
			return str_replace('____FH-percent____', '%', $mask);
		}
    }
    
    /**
     * Handler::_text()
     *
     * Private: return the given text in the correct language
     *
     * @param int $index: the index of the text in the textfile
     * @return string: the text in the correct language
     */
    function _text( $iIndex ) {
        
        // is a language set?
        if(!is_array($this->_text)) {
            trigger_error('No language file set!', E_USER_ERROR);
            return false;
        } else if(!array_key_exists($iIndex, $this->_text)) {
            trigger_error('Unknown index '.$iIndex.' to get language string!', E_USER_NOTICE);
            return '';
        }
        
        return $this->_text[$iIndex];        
    } 
    
    /**
     * Handler::_registerFileName()
     *
     * Private: Register the filenames which upload fields are using for there 
     * uploaded file so that other upload fields cannot use these.
     *
     * @param string $sFilename: the filename to register
     * @param string $sField: the field who is registering the file
     * @return bool: false if the filename is allready registered, true otherwise
     */
    function _registerFileName($sFilename, $sField) {
        static $aFilenames = array();
        
        if(isset($aFilenames[$sFilename]) && $aFilenames[$sFilename] != $sField) {
            return false;
        } 
        
        $aFilenames[$sFilename] = $sField;
        return true;
    }   
    
    /**
     * Handler::_loadDbData()
     *
     * Private: load the data when it's a edit form
     *
     * @return void
     */
    function _loadDbData() {
    	// get the data from the database
    	$sql = $this->_db->query("
    	  SELECT * FROM ".$this->_db->quote( $this->_table ) .
    	  $this->_getWhereClause() 
    	);
    	
    	// any records found
    	if( $this->_db->recordCount( $sql ) >= 1) {
    		$this->_dbData = $this->_db->getRecord( $sql );
    	} 
    	
    	// no records found, auto insert ?
    	else if(FH_AUTO_INSERT) {
    		// get the keys
    		$keys = $this->_db->getPrKeys( $this->_table ); 
    		
    		// add the primary keys values to the fields
	        $size1 = sizeof( $keys );
            $size2 = sizeof( $this->_id );
            for($i = 0; $i < ( $size1 > $size2 ? $size2 : $size1 ) ; $i++ ) {
                $this->addValue( $keys[$i], $this->_id[$i] );
            }   
            
            // change the type of form from update to insert
            $this->insert = true;
            $this->edit   = false;
        } 
        // record not found and no auto insert..
        else {
            trigger_error(
              'Try to edit a none existing record!', 
              E_USER_ERROR
            );
        }                
    }
    
    /**
     * Handler::_saveData()
     *
     * Private: save the data into the database
     *
     * @return int: the id which was used to save the record
     */
    function _saveDbData( $data ) { 
    	$notNullFields = $this->_db->getNotNullFields( $this->_table );
    	while( list( $key, $value ) = each( $notNullFields ) ) {
    		$notNullFields[$key] = strtolower($value);
    	}
    	
    	// get the data which should be saved
        foreach( $data as $field => $value ) {
            // do we have to save the field ?
            if( !in_array($field, $this->_dontSave) ) {
                
                // Is the value ''  and is it a date or timefield? Then save NULL
                // This is needed when the fields are NOT required
                if($value == '' && !in_array(strtolower($field), $notNullFields) ) {
                    $value = 'NULL';
                    $this->_sql[] = $field;                    
                }
                
                $save[$field] = $value;
            }
        } 
        
        // change the date's to the right format
        foreach( $this->_date as $field ) {
            if($save[$field] != 'NULL' && preg_match('/\d{1,2}-\d{1,2}-\d{4}/', $save[$field])) {
                $save[$field] = $this->_db->dbDate( $save[$field] );
                $this->_sql[] = $field;
            }
        }
        
        // get the query
        $query = $this->_getQuery( $save, $this->_sql, $this->edit );
        
        # debug!
        //die( $query ); 
        
        // is there something to save ?
        if( !$query ) {
        	return;
        }
        
        // execute the query
        $sql = $this->_db->query( $query );
        
        // query failed?
        if( !$sql ) 
        {
            trigger_error(
              "Error, query failed!<br />\n".
              "<b>Error message:</b> ".$this->_db->getError()."<br />\n".
              "<b>Query:</b> ". $query,
              E_USER_ERROR
            );
            return;
        } 
        else 
        {        
            if( $this->edit ) {
            	$return = $this->_id;
            } else {
            	$id = $this->_db->getInsertId( $this->_table );
            	if( $id ) { 
            		$return = $id;
            	} else {
            		$keys = $this->_db->getPrKeys( $this->_table );
            		
            		$result = array();
            		foreach( $keys as $key ) {
    	        		if( array_key_exists( $key, $data ) ) {
    	        			$result[] = $data[$key];
    	        		}
            		}
            		$size = sizeof( $result );
            		$return = $size > 1 ? $result : ( $size == 1 ? $result[0] : null);
            	}
            }
        }
        
        // unset the database object (we dont need it anymore)
        unset( $this->_db );                
                
        return $return;
    }
    
    /**
     * Handler::_getQuery()
     *
     * Generate a query from the given data and return it
     *
     * @param array $data: array of field => value  which should be saved
     * @param array $sqlFields: array of field which value is an SQL function (so it should not be quoted)
     * @param boolean $edit: do we have to generate an edit or insert query
     * @return <type>
     * @access private|public
     * @author Teye Heimans   
     */
    function _getQuery( $data, $sqlFields, $edit ) {
    	$fieldNames = $this->_db->getFieldNames( $this->_table );
    	
    	if( !$fieldNames ) {
    	    trigger_error(
    	      "Could not fetch the fieldnames of the table ". $this->_table,
    	      E_USER_WARNING
    	    );  
    	    return false;
    	}
    	
    	// walk the data from the form
        foreach( $data as $field => $value ) {
        	
        	// not in field list? remove the data (we dont need it)
            if(!in_array($field, $fieldNames)) {
                unset( $data[$field] );
            } else {
                // is the value an array? Implode it!
            	if(is_array($value)) $value = implode(', ', $value);
            	
            	// remove spaces etc
	            $value = trim( $value );
	            
	            // is the value of the field a SQL field? Then don't quote it!
                if(!in_array($field, $sqlFields) ) 
                {
                    $value = $this->_db->escapeString( $value );
                    if( preg_match('/^-?\d*\.?\d+$/', $value)) {
                        if( $this->_db->quoteNumbers() ) {                         
                            $value = "'".$value."'";
                        }
                    } else {
                        $value = "'".$value."'";
                    }
                    $data[$field] = $value;
                }
            }
        }
        
        // check if there is still something left to save...
        if( sizeof( $data ) == 0 ) {
        	return false;
        }       
        
        // generate the query
        if($edit) {
            $query = 'UPDATE '.$this->_db->quote( $this->_table ).' SET ';
            foreach($data as $field => $value) {
                $query .= ' '.$this->_db->quote( $field ).' = '.$value.', ';
            }            
            $query = substr($query, 0, -2) . $this->_getWhereClause();
        } else {
        	$query = 
            'INSERT INTO '.$this->_db->quote( $this->_table ).' (';
            foreach( array_keys($data) as $field ) { 
            	$query .= $this->_db->quote( $field ) .', ';
            }
            $query  = substr($query, 0, -2) . ') VALUES (';
            $query .= implode(', ', $data).')';
        }
        
        return $query; 
    }
    
    /**
     * Handler::_getWhereClause()
     *
     * get the where clause 
     *
     * @return string
     * @access public
     * @author Teye Heimans   
     */
    function _getWhereClause( $operator = '=', $extra = array() ) { 
    	$keys   = $this->_db->getPrKeys( $this->_table );
    	$record = $this->_id;
        
        $size1 = sizeof( $record );
        $size2 = sizeof( $keys );
        
        $data = array();
        for($i = 0; $i < ($size1 < $size2 ? $size1 : $size2); $i++ ) {
            if(preg_match("/^-?\d*\.?\d+$/", $record[$i]) ) {
                if( $this->_db->quoteNumbers() ) {                         
                    $record[$i] = "'".$record[$i]."'";
                }
            } else {
                $record[$i] = "'".$record[$i]."'";
            }
            $data[] = ' '.$this->_db->quote( $keys[$i] ) .' '.$operator.' '.$record[$i].' ';
        }
        
        $data = array_merge($data, $extra);
        
        return (sizeof($data) ? ' WHERE ' . implode(' AND ', $data) : '');
    }
    
    /**
     * Handler::_handleUploads()
     *
     * Private: method to handle the uploads and image convertions
     *
     * @return void
     */
    function _handleUploads() {        
        // upload the uploaded files
        foreach($this->_upload as $name) {
           $this->_fields[$name][1]->doUpload();
        }  
        
        foreach( $this->_convert as $field => $convertions ) {        	
            foreach( $convertions as $action => $data ) {
            	
                // is the field a uploadfield?
                if( in_array($field, $this->_upload) ) {
                    // is there a file uploaded ?
                    if($this->_fields[$field][1]->isUploaded()) {
                        $image = 
                        $this->_fields[$field][1]->getSavePath().
                        $this->_fields[$field][1]->getValue();
                        
                        
                        // does the file exists?     
                        if(!file_exists($image)) {
                            trigger_error("Error! Could not find uploaded file $image!", E_USER_WARNING);
                            unset($image);
                        }
                    } else {
                    	// image is not uploaded, continue with next field/image
                    	continue;
                    }            
                // it's not a uploadfield... is it an image ?
                } else if(file_exists($field)) {
                    $image = $field;
                // unknown field or file!    
                } else {
                    trigger_error('Could not find field or file to convert: '.$image , E_USER_WARNING);
                }
                
                // do the convert actions with the image (when the uploaded file is a jpg or png!)
                if( isset($image) && in_array( strtolower(substr( $image, -4) ), array('.jpg', '.png', 'jpeg')) ) {
                    $img =& new ImageConverter( $image );
                    
                    // stop when a error occoured
                    if( $img->getError() ) {
                        trigger_error( $img->getError(), E_USER_WARNING );
                        unset( $img );
                        continue;
                    }
                    
                    foreach($data as $info) {
                    	switch($action) {
                    	  case 'merge':
                    		list( $stamp, $align, $valign, $transparant ) = $info;                    
                    		 
                            $img->doMerge( $stamp, $align, $valign, $transparant );                		
                    		break;
                    	  case 'resize':  
                    		list( $destination, $maxX, $maxY, $quality ) = $info;
                    		if( empty( $destination ) ) {
                    			$destination = $image;
                    		}
                    		  
                            $img->setQuality( $quality );
                            $img->doResize( $destination, $maxX, $maxY );
                    		break;
                    	}
                    } 
                    
                    unset( $img );
                }
                unset( $image );   
            }
        }
    }
    
    /**
     * Handler::_getForm()
     *
     * Private: get the form
     *
     * @return string: the generated form
     */
    function _getForm( $iDisplayPage = null) {
        if( is_null( $iDisplayPage ) ) {
            $iDisplayPage = $this->_curPage;
        }
        if( $iDisplayPage <= 0) {
        	$iDisplayPage = 1;
        }
        
        // set the tab indexes for the fields...        
        reset( $this->_tabindexes );
        ksort( $this->_tabindexes );
        while( list( $index, $field ) = each( $this->_tabindexes )) {
            if( $this->fieldExists( $field ) ) {                
                $this->_fields[$field][1]->setTabIndex( $index );                
            } else {
                trigger_error(
                  'Error, try to set the tabindex of an unknown field "'.$field.'" !'
                );
            }
        }
        
        // set the focus to the first (tab index) field if no focus is set yet 
		if( is_null($this->_focus)) {
		    // are there tab indexes set ?
		    if( sizeof( $this->_tabindexes) > 0 ) {
		        // set the focus to the element with the lowest positive tab index
    		    reset( $this->_tabindexes );
    		    while( list( $key, $field ) = each( $this->_tabindexes )) {
    		        if( $key >= 0 && $this->setFocus( $field )) {
    		            break;
    		        }		            
    		    }    		
		    } 
		    
		    // no focus set yet. Set the focus to the first field    
		    if( is_null($this->_focus)) {
		        foreach( $this->_fields as $name => $data ) {
		            // is it a object (only fields + buttons are objects)
		            if( is_object( $this->_fields[$name][1]) && $this->setFocus( $name )) {		
                        break;
		            }
		        }
		    }		        
		}
        
        // used vars
        $hidden = '';
        $form   = '';
        $buffer = array();
        $repeat = true;
        $page   = 1;
        
        // walk trought the fields array
        foreach( $this->_fields as $id => $field ) 
        {    
            switch( $field[0] ) {        	    
        	    // multiple pages in this form
        	    case '__PAGE__':
        	      if( $field[1] == $iDisplayPage) {
        	          break 2;
        	      } 
        	      $page++;
        	      break;
        	      
        		// hidden field
        		case '__HIDDEN__':
        		  $hidden .= $field[1]->getField()."\n";                
        		  break;
        		  
        		// new mask to set  
        		case '__MASK__':
                  list($this->_mask, $repeat) = $field[1];
                  break;
                  
                // insert html or a line
                case '__HTML__':
                case '__LINE__':
                  if($page == $iDisplayPage ) { 
      				  $form .= $field[1]; 
  				  }
                  break;
                  
                // begin new fieldset  
                case '__FIELDSET__':
                  if($page == $iDisplayPage ) { 
				      array_push( $buffer, array($form, $field[1][0], $field[1][1]) ); 
				      $form = ''; 
				  } 
				  break;
                  
                // end new fieldset  
                case '__FIELDSET-END__':
                  if($page == $iDisplayPage ) { 
				      if(sizeof($buffer) > 0) { 
				          $d = array_pop($buffer); 
				          $form = $d[0]. 
				          str_replace( 
				            array('%name%', '%caption%', '%content%' ), 
				            array($d[1], $d[2], $form ), 
				            FH_FIELDSET_MASK 
				          ); 
				         } else { 
				          trigger_error('Fieldset is closed while there is not an open fieldset!'); 
				      } 
				  } 
				  break;    
                
                // default action: field or button    
                default:
                  if($page != $iDisplayPage ) {
                      // put the data of the field in a hidden field
                      if($field[0] != '__BUTTON__') {
                          $h =& new HiddenField( $this, $id );                     
        	              $h->setValue( $field[1]->getValue() );                         
        	              $hidden .= $h->getField() ."\n";
        	              unset( $h );
        	          }      
                  } else {
                      // field is on the current page.. just display it
                      $html = $this->_loadMask( $field, $this->_mask, $id );
                      if($html) {
                          $form .= $html;
                          
                          if(!$repeat) {
                          	  $this->_mask = FH_DEFAULT_ROW_MASK;
                          } else if(is_numeric($repeat)) {
                              $repeat--;
                          }
                      }
                  }
                  break;
            }
        }
        
        // give the form a page number...
        if($this->_pageCounter > 1) {
            $h =& new HiddenField( $this, $this->_name .'_page' );
            $h->setValue( $iDisplayPage );
            $hidden .= $h->getField() ."\n";
            unset( $h );            
        }
        	          
        // get a possible half filled mask
        $form .= $this->_loadMask( null, null, null );
        
        // get errors
        $errors =& catchErrors();
        $errmsg = '';
        foreach($errors as $error) {
        	switch ($error['no']) {
        		case E_USER_WARNING: $type = 'Warning'; break;
        		case E_USER_NOTICE:  $type = 'Notice';  break;
        		case E_USER_ERROR:   $type = 'Error';   break;
        		default: 			 $type = 'Warning ('.$error['no'].')'; break;
        	}
            $errmsg .= "<b>".$type.":</b> ".basename($error['file'])." at ".$error['line']." ". $error['text'] ."<br />\n";
        }
        
        // NOTE!!
        // DO NOT REMOVE THIS!
        // You can remove the line "This form is generated by FormHandler" in the config file!!
        // DONT REMOVE THE HTML CODE BELOW! Just set FH_EXPOSE to FALSE!
        $result =
        $errmsg .        
        "<!--\n".
        "  This form is automaticly being generated by FormHandler v3.\n".
        "  See for more info: http://www.formhandler.net\n".
        "  This credit MUST stay intact for use\n".
        "-->\n".
        $this->getJavascriptCode(). 
        "<form name='".$this->_name."' method='post' action='".$this->_action."'".
        (sizeof($this->_upload) ? ' enctype="multipart/form-data"':'' ).
        ($this->_extra ? " ".$this->_extra : "" ).">\n".
        $hidden.
        ($this->_setTable ? "<table border='0' cellspacing='0' cellpadding='3'>\n" : '' ).
        $form.
        ($this->_setTable ? "\n</table>\n" : '').
        (FH_EXPOSE ? 
          "<small><font color='B5B5B5' face='tahoma' style='font-family:tahoma;font-size:10px;color:B5B5B5;font-weight:normal;'>".
          "This form is generated by </font></small><a href='http://www.formhandler.net' target='_blank'><small>".
          "<font color='B5B5B5' face='tahoma' style='font-family:Tahoma;font-size:10px;color:B5B5B5;font-weight:normal;'><b>FormHandler</b></font></small></a>" :''
        ).     
        "</form>\n".     
        "<!--\n".
        "  This form is automaticly being generated by FormHandler v3.\n".
        "  See for more info: http://www.formhandler.net\n".
        "-->";
        $js = '';
        
        // load the first dynamic selectfield if this is used
        if( isset( $this->_dynamicLoader ) ) {
        	$js .= $this->_dynamicLoader;
        }
        
        // when the focused is set...
        if($this->_focus) { 
            $js .=	
            "// set the focus on a specific field \n".
          	"if( document.getElementById && document.getElementById('".$this->_focus."') ) {\n".
          	"    elem = document.getElementById('".$this->_focus."');\n".
          	"    if( elem.type != 'hidden') {\n".
          	"        try {\n".
          	"          elem.focus();\n".
            "        } catch(e) {}\n". 
          	"    }\n".
          	"}\n";
        }          
        
        if( !empty($js) ) {
        	$result .= 
        	"\n<script language='javascript' type='text/javascript'>\n".
        	"<!-- //\n". $js ."//-->\n".
            "</script>";
        }
               
        
        return $result;
    }	
}

?>